Package: moanin
Title: An R Package for Time Course RNASeq Data Analysis
Version: 1.18.0
Authors@R: c(person("Elizabeth", "Purdom", email="epurdom@stat.berkeley.edu",
                    role=c("aut"),
                    comment=c("orcid"="https://orcid.org/0000-0001-9455-7990")),
             person("Nelle", "Varoquaux", role=c("aut", "cre"),
                    email="nelle.varoquaux@gmail.com",
                    comment=c(orcid="https://orcid.org/0000-0002-8748-6546")))
Description: Simple and efficient workflow for time-course gene expression
	     data, built on publictly available open-source projects hosted on
	     CRAN and bioconductor. moanin provides helper functions for all
	     the steps required for analysing time-course data using
	     functional data analysis: (1) functional modeling of the
	     timecourse data; (2) differential expression analysis; (3)
	     clustering; (4) downstream analysis.
Depends: R (>= 4.0), SummarizedExperiment, topGO, stats
Imports: S4Vectors, MASS (>= 1.0.0), limma, viridis, edgeR, graphics,
        methods, grDevices, reshape2, NMI, zoo, ClusterR, splines,
        matrixStats
Suggests: testthat (>= 1.0.0), timecoursedata, knitr, rmarkdown,
        markdown, covr, BiocStyle
VignetteBuilder: knitr
License: BSD 3-clause License + file LICENSE
Encoding: UTF-8
LazyData: false
RoxygenNote: 7.1.1
biocViews: TimeCourse, GeneExpression, RNASeq, Microarray,
        DifferentialExpression, Clustering
git_url: https://git.bioconductor.org/packages/moanin
git_branch: RELEASE_3_22
git_last_commit: 6b72807
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:10:47 UTC; biocbuild
Author: Elizabeth Purdom [aut] (ORCID: <https://orcid.org/0000-0001-9455-7990>),
  Nelle Varoquaux [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8748-6546>)
Maintainer: Nelle Varoquaux <nelle.varoquaux@gmail.com>

# Generated by roxygen2: do not edit by hand

export(consensus_matrix)
export(create_go_term_mapping)
export(discont_basis)
export(find_enriched_go_terms)
export(get_auc_similarity_scores)
export(perWeek_barplot)
export(plot_cdf_consensus)
export(plot_model_explorer)
export(pvalues_fisher_method)
exportClasses(Moanin)
exportMethods("[")
exportMethods("group_variable<-")
exportMethods("group_variable_name<-")
exportMethods("time_variable<-")
exportMethods("time_variable_name<-")
exportMethods(DE_timecourse)
exportMethods(DE_timepoints)
exportMethods(basis_matrix)
exportMethods(create_moanin_model)
exportMethods(create_timepoints_contrasts)
exportMethods(degrees_of_freedom)
exportMethods(estimate_log_fold_change)
exportMethods(get_log_data)
exportMethods(group_variable)
exportMethods(group_variable_name)
exportMethods(log_transform)
exportMethods(plot_splines_data)
exportMethods(rescale_values)
exportMethods(show)
exportMethods(spline_formula)
exportMethods(splines_kmeans)
exportMethods(splines_kmeans_predict)
exportMethods(splines_kmeans_score_and_label)
exportMethods(time_by_group_variable)
exportMethods(time_variable)
exportMethods(time_variable_name)
import(SummarizedExperiment)
import(methods)
importClassesFrom(S4Vectors,DataFrame)
importFrom(ClusterR,KMeans_rcpp)
importFrom(edgeR,DGEList)
importFrom(edgeR,calcNormFactors)
importFrom(grDevices,rainbow)
importFrom(graphics,axis)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(limma,contrasts.fit)
importFrom(limma,eBayes)
importFrom(limma,lmFit)
importFrom(limma,voom)
importFrom(methods,new)
importFrom(splines,ns)
importFrom(stats,as.formula)
importFrom(topGO,GenTable)
importFrom(topGO,annFUN.gene2GO)
importFrom(topGO,runTest)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(viridis,viridis)

setGeneric("splines_kmeans", 
           function(object,...) { standardGeneric("splines_kmeans")})
setGeneric("splines_kmeans_score_and_label", 
           function(object,...) { standardGeneric("splines_kmeans_score_and_label")})
setGeneric("splines_kmeans_predict", 
           function(object,...) { standardGeneric("splines_kmeans_predict")})


#' Performs splines clustering using K-means
#'
#' @inheritParams DE_timecourse
#' @param n_clusters int optional, default: 10
#' @param init  ["kmeans++", "random", "optimal_init"]
#' @param n_init int, optional, default: 10
#'      Number of initialization to perform.
#' @param max_iter  int, optional, default: 300
#'  Maximum number of iteration to perform  
#' @param random_seed int, optional, default: NULL. 
#' Passed to argument \code{seed} in  \code{\link[ClusterR]{KMeans_rcpp}}. 
#' If NULL (default), set to \code{.Random.seed[1]}.
#' @param fit_splines   boolean, optional, default: TRUE
#'  Whether to fit splines or not.
#' @param rescale   boolean, optional, default: TRUE
#'  Whether to rescale the data or not.
#' @details If \code{\link{Moanin}} object's slot has log_transform=TRUE, then the data will be transformed by the function log(x+1) before applying splines and clustering.
#' @return A list in the format returned by \code{\link[ClusterR]{KMeans_rcpp}},
#'    with the following elements added or changed:
#'\itemize{
#'\item{\code{centroids}}{ The centroids are rescaled so that they range from
#'0-1}
#'\item{\code{fit_splines}}{ Logical, the value of \code{fit_splines} given to the
#'function }
#'\item{\code{rescale}}{ The value of \code{rescale} given to the function }
#'} 
#' @name splines_kmeans
#' @aliases splines_kmeans,Moanin-method
#' @examples 
#' data(exampleData)
#' # Use the default options
#' moanin <- create_moanin_model(data=testData, meta=testMeta)
#' out <- splines_kmeans( moanin,n_clusters=5)
#' table(out$clusters)
#' @importFrom ClusterR KMeans_rcpp
#' @export
setMethod("splines_kmeans", "Moanin",
    function(object, n_clusters=10,
                        init="kmeans++",
                        n_init=10, 
                        max_iter=300, 
                        random_seed=.Random.seed[1],
                        fit_splines=TRUE,
                        rescale=TRUE){
    basis <- basis_matrix(object)
    data <- get_log_data(object)
    if(fit_splines){
        fitted_data <- fit_predict_splines(data=data, object)
    }else{
        fitted_data <- data
    }
    
    ## CHECK ME: previous version gave the meta information to rescale_values, 
    ## but also group=NULL, so meant it wasn't used. Was that a mistake?
    if(rescale){
        fitted_data <- rescale_values(data=fitted_data, object=object,
                                     use_group=FALSE)
    }
    
    kmeans_clusters <- ClusterR::KMeans_rcpp(
        fitted_data, n_clusters, num_init=n_init, max_iters=max_iter,
        seed=random_seed, initializer=init)
    if(any(kmeans_clusters$obs_per_cluster == 0)){
        warning(
            paste(
                "splines_kmeans: Empty cluster found. Consider increasing ",
                "the number of initialization or using a better ",
                "initialization strategy"))}
    kmeans_clusters$centroids <- rescale_values(
        data=kmeans_clusters$centroids, object=object, use_group=FALSE)
    names(kmeans_clusters$clusters) <- row.names(object)
    
    # Give names to clusters
    cluster_names <- vapply(seq_len(n_clusters), FUN=function(x){paste0("C", x)},
                        FUN.VALUE="C")
    row.names(kmeans_clusters$centroids) <- cluster_names
    colnames(kmeans_clusters$centroids) <- colnames(object)
    
    #kmeans_clusters$moanin_model <- moanin_model
    kmeans_clusters$fit_splines <- fit_splines
    kmeans_clusters$rescale <- rescale
    return(kmeans_clusters)
}
)

#' @param method If "distance", predicts based on distance of data to kmeans
#'   centroids. If "goodnessOfFit", is a wrapper to
#'   \code{splines_kmeans_score_and_label}, assigning labels based on goodness
#'   of fit, including any filtering.
#' @param ... arguments passed to \code{splines_kmeans_score_and_label}
#' @return \code{splines_kmeans_predict} returns a vector giving the labels for
#'   the given data.
#' @rdname splines_kmeans_score_and_label
#' @export
setMethod("splines_kmeans_predict", "Moanin",
          function(object, kmeans_clusters, data=NULL, 
                   method=c("distance", "goodnessOfFit"), ...){
    method <- match.arg(method)
    if(method == "goodnessOfFit"){
        out <- splines_kmeans_score_and_label(
            object=object, 
            kmeans_clusters=kmeans_clusters, data=data, ...)
        return(out$labels)
    }
    if(method == "distance"){
        fit_splines <- kmeans_clusters$fit_splines
        rescale <- kmeans_clusters$rescale
        check_data_meta(kmeans_clusters$centroids, object)
        basis <- basis_matrix(object)
        if(is.null(data)){
            data <- get_log_data(object)}
        else{
            check_data_meta(data, object)
        }
        if(fit_splines){
            fitted_data <- fit_predict_splines(data=data, moanin_model=object)
        }else{
            fitted_data <- data
        }
        
        if(rescale){
            fitted_data <- rescale_values(data=fitted_data, object=object, 
                                          use_group=FALSE)
        }
        
        closest_cluster <- function(x) {
            cluster_dist <- apply(
                kmeans_clusters$centroids, 1, function(y){sqrt(sum((x-y)^2))})
            return(which.min(cluster_dist)[1])
        }
        
        all_labels <- apply(fitted_data, 1, closest_cluster) 
        kmeans_clusters$clusters <- all_labels
        names(kmeans_clusters$clusters) <- row.names(data)
        return(kmeans_clusters$clusters)
    }

}
)


#' Assign score and labels from raw data
#' @name splines_kmeans_score_and_label
#' @param object the Moanin object that contains the basis functions used in
#'   creating the clusters
#' @param kmeans_clusters the results of running \code{\link{splines_kmeans}}
#' @param data the data to predict. If not given, will use \code{assay(object)}.
#'   If given, the number of columns of \code{data} must match that of
#'   \code{object}
#' @param kmeans_clusters  List returned by \code{\link{splines_kmeans}}
#' @param proportion_genes_to_label float, optional, default: 0.5
#'  Percentage of genes to label. If max_score is provided, will label
#'  genes that are either in the top `proportion_genes_to_label` or with a
#'  score below `max_score`.
#' @param max_score optional, default: Null
#'  When provided, will only label genes below that score. If NULL, ignore
#'  this option.
#' @param previous_scores matrix of scores, optional. Allows user to give the
#'   matrix scores results from a previous run of
#'   \code{splines_kmeans_score_and_label}, and only redo the filtering (i.e. if
#'   want to change \code{proportion_genes_to_label} without rerunning the
#'   calculation of scores)
#' @param rescale_separately logical, whether to score separately within
#'   grouping variable
#' @return A list consisting of
#' \itemize{
#' \item{\code{labels}}{the label or cluster assigned to each gene based on the
#' cluster with the best (i.e. lowest) score, with no label given to genes that
#' do not have a score lower than a specified quantity}
#' \item{\code{scores}}{the matrix of size n_cluster x n_genes, containing for 
#' each gene and each cluster, the goodness of fit score}
#' \item{\code{score_cutoff}}{The required cutoff for a gene receiving an
#' assignment}
#' }
#' @aliases splines_kmeans_predict splines_kmeans_predict,Moanin-method 
#' @aliases splines_kmeans_score_and_label,Moanin-method
#' @examples 
#' data(exampleData)
#' moanin <- create_moanin_model(data=testData, meta=testMeta)
#' # Cluster on a subset of genes
#' kmClusters=splines_kmeans(moanin[1:50,],n_clusters=3)
#' # get scores on all genes
#' scores_and_labels <- splines_kmeans_score_and_label(object=moanin, kmClusters)
#' head(scores_and_labels$scores)
#' head(scores_and_labels$labels)
#' # should be same as above, only just the assignments
#' predictLabels1 <- splines_kmeans_predict(object=moanin, kmClusters, 
#'      method="goodnessOfFit")
#' # Instead use distance to centroid:
#' predictLabels2 <- splines_kmeans_predict(object=moanin, kmClusters, 
#'      method="distance")
#' @export
setMethod("splines_kmeans_score_and_label", "Moanin",
          function(object, kmeans_clusters, data=NULL,
                   proportion_genes_to_label=0.5,
                   max_score=NULL, previous_scores=NULL,
                   rescale_separately=FALSE){
    if(is.null(data)){
        data <- get_log_data(object)
    }else{
        data <- as.matrix(data)
        if(ncol(data) != ncol(kmeans_clusters$centroids)){
            stop(
                "User-given data and kmeans resultsare inconsistent. Data is has ", 
                ncol(data),"columns; kmeans result was run on", 
                ncol(kmeans_clusters$centroids), "samples")        
        }
    }

    if(!is.null(max_score) && is.na(max_score)){
        stop(
            "User-given max_score is NA. Expecting float.")
    }

    if(is.null(previous_scores)){
        n_clusters <- dim(kmeans_clusters$centroids)[1]
        all_scores <- matrix(NA, nrow=dim(data)[1], ncol=n_clusters)


        for(k in seq_len(n_clusters)){
            # By default, should not rescale separately on any columns.
            if(!rescale_separately){
                scores <- score_genes_centroid(
                    data,
                    kmeans_clusters$centroids[k,],
                    scale=FALSE)

                all_scores[, k] <- scores /  max(scores)
            }else{
                scores <- NULL
                groups <- levels(group_variable(object))
                for(group in groups){
                    mask <- group_variable(object) == group
                    partial_scores <- score_genes_centroid(
                        data[, mask],
                        kmeans_clusters$centroids[k, mask],
                        scale=FALSE)
                    if(is.null(scores)){
                        scores <- partial_scores
                    }else{
                        scores <- scores + partial_scores
                    }
                }
                all_scores[, k] <- scores / max(scores)
            }
        }

        # Give names to rows
        all_scores <- as.matrix(all_scores)
        row.names(all_scores) <- row.names(data) 
    }
    else{
        all_scores <- previous_scores
    }
    
    # If a centroid is NA, ignore it. 
    scores <- apply(all_scores, 1, function(x){min(x, na.rm=TRUE)})
    labels <- apply(all_scores, 1, which.min)
    names(labels) <- row.names(data)

    if(proportion_genes_to_label<1 | !is.null(max_score)){
        max_score_data <- stats::quantile(scores, c(proportion_genes_to_label))
        if(!is.null(max_score)){
            max_score <- min(max_score_data, max_score)
        }else{
            max_score <- max_score_data
        }
        genes_to_not_consider <- scores >= max_score
        labels[genes_to_not_consider] <- NA
    
        if(max_score == 1){
            msg <- paste(
                "moanin::splines_kmeans_score_and_label is labeling genes",
                " with a score of 1. This implies that no good fit for ",
                "those genes are found and the assignment is random.",
                sep="")
            warning(msg)
        } 
    }# Only assign labels to X% of the genes


    return(list("labels"=labels,
                "scores"=all_scores,
                "score_cutoff"=max_score))
}
)

setGeneric("rescale_values", function(object,...) { 
    standardGeneric("rescale_values")})

#' Fit splines to each gene of data matrix, used by DE_timecourse
#' @param design_matrix design matrix (containing evaluated basis matrix, but potentially other variables) to fit
#' @param data a matrix of data to fix splines to
#' @param weights A matrix of weights, of the same dimension as \code{data}.
#' @details Needed to allow for fitting weights, in which case for loop over every row/gene. Otherwise, just a call to lm.fit.
#' @return matrix of the coefficients for each basis function, each row of the
#'   matrix containing the coefficients for the corresponding gene in
#'   \code{data}.
#' @keywords internal
fit_splines <- function(design_matrix, data, weights=NULL){
    n <- ncol(design_matrix)
    nr <- nrow(data)
    if(inherits(data,"DataFrame")) data<-data.matrix(data)
    
    if(!is.null(weights)){
        beta <- matrix(nrow=nr, ncol=n)
        ##FIXME: can't we multiply data by weights and then make call to lm.fit -- would be faster than looping over all genes, no?
        for(i in seq_len(nr)){
            beta[i,] <- stats::lm.wfit(design_matrix, data[i,], weights[i,])$coefficients
        }
        row.names(beta) <- row.names(data)
    }else{
        beta <- t(stats::lm.fit(design_matrix, t(data))$coefficients)
    }
    return(beta)
}

#' Get fitted values for splines for each gene
#'
#' @inheritParams DE_timecourse
#' @param meta_prediction 
#' @param data a matrix with data (required doesn't pull from moanin_model)
#' @return a matrix of the fitted y values, with dimensions the same as
#'   \code{data}
#'
#' @keywords internal
fit_predict_splines <- function(data, moanin_model, 
                                meta_prediction=NULL){
    basis <- basis_matrix(moanin_model)
    gpVar <- group_variable_name(moanin_model)
    tpVar <- time_variable_name(moanin_model)

    if(inherits(data,"DataFrame")){
        data <- data.matrix(data)
    }

    if(is.null(meta_prediction)){
        y_fitted <- t(stats::lm.fit(basis, t(data))$fitted.values)
    }else{
        degrees_of_freedom <- degrees_of_freedom(moanin_model)
        fitting_data <- t(as.matrix(data))
        formula_data <- list(
            "Group"=group_variable(moanin_model),
            "Timepoint"=time_variable(moanin_model),
            "fitting_data"=fitting_data,
            "degrees_of_freedom"=degrees_of_freedom(moanin_model))

        names(formula_data)[c(1, 2)] <- c(gpVar, tpVar)
        updated_formula <- stats::update(spline_formula(moanin_model), 
                                        fitting_data ~ .)
        model <- stats::lm(updated_formula, formula_data)
        y_fitted <- stats::predict(model, meta_prediction)
    }
    return(y_fitted)
}


#' Create prediction meta data from splines model
#'
#' @inheritParams DE_timecourse
#' @param num_timepoints integer, optional, default: 100.
#'  Number of timepoints to use for the prediction metadata
#'
#' @return moanin_model
#' @keywords internal
create_meta_prediction <- function(moanin_model, num_timepoints=100){
    # Create moanin_model for prediction
    timepoints_pred <- NULL
    groups_pred <- NULL
    gpVar <- group_variable_name(moanin_model)
    tpVar <- time_variable_name(moanin_model)
    groups <- levels(droplevels(group_variable(moanin_model)) )
    
    # Check that the moanin model has the appropriate information to create a
    # smooth prediction model.
    if(is.null(spline_formula(moanin_model)) | 
       is.null(degrees_of_freedom(moanin_model))){
        msg <- paste(
            "Smooth prediction is not possible without the formula.",
            "Will only predict on initial points")
        warning(msg)
        return(moanin_model)
    }
    
    
    for(group in groups){
        mask <- group_variable(moanin_model) == group
        time <- time_variable(moanin_model)[mask]
        
        timepoints_pred <- c(
            timepoints_pred,
            seq(min(time), max(time), length=100))
        groups_pred <- c(
            groups_pred,
            rep(group, 100))
    }
    meta_prediction <- data.frame(
        "Timepoint"=timepoints_pred,
        "Replicates"=rep(1, length(timepoints_pred)),
        "Group"=groups_pred)
    names(meta_prediction)[c(1,3)]<-c(tpVar,gpVar)
    return(meta_prediction)
}


#' Rescales rows of data to be between 0 and 1
#'
#' @param data The matrix to rescale by row. If NULL, and \code{object} is
#'   given, data will be taken as \code{assay(object)} Each row should
#'   correspond to a gene or a centroid, and columns to samples.
#' @param object a object of class Moanin, only needed if choose to rescale by
#'   grouping variable in the moanin object. If NULL, then data will be rescaled
#'   jointly across all observations.
#' @param use_group If true, then the data will be rescaled such that, for each
#'   row, all values associated to each group (defined by grouping variable of
#'   \code{object}) is between 0 and 1. For example, if column
#' @param ... arguments passed to the matrix or Moanin method.
#' @details If the user set \code{log_transform=TRUE} in the creation of the
#'   \code{Moanin} object, the data will be log transformed before rescaling
#' @return rescaled y, such that for each row, the values are comprised between
#'   0 and 1. Note that if \code{use_group=TRUE} and \code{object} is not NULL,
#'   the values associated to the columns of unique values of the grouping
#'   variable of \code{object} will be rescaled separately.
#' @export
#' @name rescale_values
#' @examples 
#' data(exampleData)
#' moanin <- create_moanin_model(data=testData, meta=testMeta)
#' # Can rescale data in Moanin object
#' allData <- rescale_values(moanin)
#' # Or provide different data and/or rescale within grouping variable
#' smallData <- rescale_values(moanin, data=testData[1:10,], use_group=TRUE)
#' @aliases rescale_values,Moanin-method
setMethod("rescale_values", "Moanin",
    function(object, data=NULL, use_group=FALSE){
    if(is.null(data)){
        data <- get_log_data(object)
    }
    if(use_group){
        if(is.data.frame(data) || inherits(data, "DataFrame")){
            data <- data.matrix(data)
        }
        factors_to_consider <- levels(group_variable(object))
        for(factor in factors_to_consider){
            mask <- group_variable(object) == factor
            ymin <- matrixStats::rowMins(data[, mask]) 
            data[, mask] <- data[, mask] - ymin
            ymax <- matrixStats::rowMaxs(data[, mask])
            whNonZero <- which(ymax>0)
            data[whNonZero,mask] <- data[whNonZero,mask] / ymax[whNonZero]
        }
        return(data)
    } else {
        return(rescale_values(object=NULL, data=data))
    }
})

#' @aliases rescale_values, NULL-method
#' @export
#' @rdname rescale_values
setMethod("rescale_values", "NULL",
    function(object, data){
        if(inherits(data, "DataFrame")){
            data <- data.matrix(data)
        }
        ymin <- matrixStats::rowMins(data) 
        data <- data - ymin
        ymax <- matrixStats::rowMaxs(data)
        whNonZero <- which(ymax > 0)
        if(length(whNonZero) > 0){
            data[whNonZero,] <- data[whNonZero,] / ymax[whNonZero]
        }
        return(data)
})

#' @aliases rescale_values,missing-method
#' @export
#' @rdname rescale_values
setMethod("rescale_values", "missing",
          function(object, ...){
              rescale_values(object=NULL, ... )
          })

# Note that if would estimate a negative scaling factor, sets it to zero 
# instead, so that your centroid estimate for the data is a flat line at the mean
align_data_onto_centroid <- function(data, centroid, positive_scaling=TRUE, 
            returnType=c("data","centroid")){
    returnType <- match.arg(returnType)
    n_samples <- dim(data)[2]
    n_genes <- dim(data)[1]
    if(n_samples != length(centroid)){
        stop(
            paste0("align_data_onto_centroid: problem in dimensions. There are ",
                   n_samples, "but centroid is of length", length(centroid)))
    }
    
    # No clue why sometimes the vector/matrix is not numeric
    if(!is.numeric(centroid)){
        centroid <- as.numeric(centroid)
    }
    centered_centroid <- centroid - mean(centroid)
    # Identify if some rows of the data are only zeros.
    only_zero_genes <- matrixStats::rowSums2(abs(data)) == 0
    scaling_factors <- apply(
        data, 1,
        function(x){sum(centered_centroid * x)/sum((x - mean(x))*x)}) 
    if(positive_scaling){
        if(returnType=="data"){
            scaling_factors[scaling_factors < 0] <- 0
        }else{
            scaling_factors <- abs(scaling_factors)
        }
    }
    
    # Now replace the scaling factors of only 0 genes by 0
    scaling_factors[only_zero_genes] <- 0
    
    # Estimate the shift factors now
    shift_factors <- apply(
        scaling_factors * data,
        1,
        function(x) mean(centroid - x)) 
    if(returnType == "data"){
        data_fitted <- (scaling_factors * data + shift_factors)        
    }
    else{
        data_fitted <- matrix(centroid,
                              nrow=nrow(data),
                              ncol=length(centroid),
                              byrow=TRUE)
        data_fitted <- sweep(data_fitted,1,shift_factors,"-")
        whZero <- which(scaling_factors == 0)
        if(length(whZero) > 0){
            # If scaling factor is zero, then centroid estimate is just a flat 
            # line equal to the mean of the centroid - shift factor. 
            data_fitted[-whZero,] <- sweep(data_fitted[-whZero,,drop=FALSE],1,
                                          scaling_factors[-whZero],"/")
            data_fitted[whZero,] <- matrix(
                matrixStats::rowMeans2(data[whZero,,drop=FALSE]),
                nrow=length(whZero),
                ncol=ncol(data_fitted), byrow=FALSE)         
        }
        else 
            data_fitted <- sweep(data_fitted, 1, scaling_factors, "/")
        
    }
    return(data_fitted)
}


score_genes_centroid <- function(data, centroid, positive_scaling=TRUE, 
                                 scale=TRUE){
    n_genes <- dim(data)[1]
    centroid <- as.numeric(centroid)
    
    data_fitted <- align_data_onto_centroid(
        data, centroid, positive_scaling=positive_scaling)
    
    scores <- apply(
        data_fitted,
        1,
        function(y) sum((centroid - y)**2))
    
    if(scale){
        all_zeros_gene <- matrix(0, 1, dim(data)[2])
        all_zeros_gene <- align_data_onto_centroid(
            all_zeros_gene,
            centroid,
            positive_scaling=positive_scaling)
        score <- sum((centroid - all_zeros_gene)**2)
        max_score <- score
    }else{
        max_score <- 1
    }
    return(scores / max_score)
}


#' Fisher's method to combine pvalues
#'
#' Combines all p-values per rows.
#' 
#' @param pvalues a matrix of pvalues, with columns corresponding to different
#'   tests or sources of p-values, and rows corresponding to the genes from
#'   which the p-values come.
#' @return a vector of p-values, one for each row of \code{pvalues}, that is the
#'   result of Fisher's combined probability test applied to the p-values in
#'   that row.
#' @examples 
#' data(exampleData)
#' moanin <- create_moanin_model(data=testData,meta=testMeta)
#' contrasts <- create_timepoints_contrasts(moanin,"C", "K")
#' deTimepoints=DE_timepoints(moanin, 
#'   contrasts=contrasts, use_voom_weights=FALSE)
#' fisherPval=pvalues_fisher_method(
#'   deTimepoints[,grep("pval",colnames(deTimepoints))])
#' head(fisherPval)
#' @export
pvalues_fisher_method <- function(pvalues){
    # TODO Add a check that all pvalues are "valid"
    keep <- (pvalues >= 0) & (pvalues <= 1)
    pvalues[pvalues == 0] <- 1e-285
    
    lnp <- data.matrix(log(pvalues))
    chisq <- (-2) * matrixStats::rowSums2(lnp)
    df <- 2 * dim(lnp)[2]
    fisher_pval <- stats::pchisq(chisq, df, lower.tail=FALSE)
    return(fisher_pval)
}

data(exampleData)
context("moanin::splines.R")

test_that("splines::fit_predict_splines", {

    moanin_model = create_moanin_model(data=testData,meta=testMeta)
    meta_prediction = create_meta_prediction(moanin_model)

    expect_silent(fit_predict_splines(testData, moanin_model))
    expect_silent(fit_predict_splines(testData, moanin_model,
    				      meta_prediction=meta_prediction))

})

test_that("splines:align_data_onto_centroid", {
    set.seed(42)
    n_samples = 20
    n_genes = 5
    centroid = runif(n_samples)

    ## don't do anything, just make data equal to centroid.
    ## (Shouldn't do anything)
    data = rep(centroid, each=n_genes)
    dim(data) = c(n_genes, n_samples)
    expect_equal(data, align_data_onto_centroid(data, centroid))
    expect_equal(data, align_data_onto_centroid(data, centroid, returnType="centroid"))
    
    # Ok, now let's make this a bit more complicated.
    shift_param = runif(n_genes)
    scale_param = 1 + runif(n_genes)
    scale_param = rep(scale_param, times=n_samples)
    dim(scale_param) = dim(data)
    shift_param = rep(shift_param, times=n_samples)
    dim(shift_param) = dim(data)

    shifted_scaled_data = scale_param * data + shift_param
    shifted_scaled_centroid = scale_param * centroid + shift_param
    expect_equal(data,
		 align_data_onto_centroid(shifted_scaled_data, centroid))
    expect_equal(shifted_scaled_data,
                 align_data_onto_centroid(shifted_scaled_data, centroid,returnType="centroid"))

    # Check error checking works
    expect_error(align_data_onto_centroid(data[, 1:10], centroid))
})

test_that("splines:score_genes_centroid", {
    set.seed(42)
    n_samples = 20
    n_genes = 5
    centroid = runif(n_samples)

    shift = runif(n_genes)
    scale = 1 + runif(n_genes)

    data = rep(centroid, each=n_genes)
    dim(data) = c(n_genes, n_samples)
    expect_silent(score_genes_centroid(data, centroid))
    expect_equal(sum(score_genes_centroid(data, centroid)), 0)

})

test_that("splines:rescale_values", {
    moanin_model = moanin::create_moanin_model(data=testData,meta=testMeta)
    expect_silent(rescaled_data <- rescale_values(moanin_model,use_group=FALSE))
    expect_silent(rescaled_data <- rescale_values(moanin_model,data=moanin:::get_log_data(moanin_model),use_group=FALSE))
    expect_equal(rep(0, nrow(moanin_model)), as.vector(rowMins(rescaled_data)))
    expect_equal(rep(1, nrow(moanin_model)), as.vector(rowMaxs(rescaled_data)))

    #check different imputs
    expect_silent(rescaled_data2<- rescale_values(moanin_model,
                  data=get_log_data(moanin_model)[1:20,],use_group=FALSE))
    expect_equal(rescaled_data[1:20,],rescaled_data2)    
    expect_silent(rescaled_data3<- rescale_values(object=NULL,
                    data=assay(moanin_model)[1:20,]))
    expect_equal(rescaled_data3,rescaled_data2)    

})


test_that("splines::create_meta_prediction", {
    moanin_model = create_moanin_model(data=testData,meta=testMeta)
    expect_silent(create_meta_prediction(moanin_model))

    # Recreate moanin model without providing the formula
    # (will create warning in create_meta_prediction)
    basis = basis_matrix(moanin_model)
    moanin_model = create_moanin_model(data=testData,meta=testMeta, basis=basis)
    expect_warning(create_meta_prediction(moanin_model))
})

# Normalization utility functions
#

#' Utility function to filter out low-expressed genes
#'
#' @param counts n by p matrix containing the count data.
#' @param min_counts integer, corresponding to the minimum number of counts for
#'  the gene to be considered expressed in a sample. Default is 20.
#' @param min_samples integer, corresponding to the minimum of samples for a
#'  gene to be expressed to be included in downstream analyses.
#' @return The filtered counts matrix
#' @keywords internal
expression_filtering <- function(counts, min_counts=20, min_samples=3){
    rows_to_keep <- apply(counts, 1, function(x){
        sum(x > min_counts) > min_samples
    })
    counts <- counts[rows_to_keep, ]
    return(counts)
}

setGeneric("log_transform", 
           function(object) { standardGeneric("log_transform")})
setGeneric("get_log_data", 
           function(object) { standardGeneric("get_log_data")})
setGeneric("degrees_of_freedom", 
    function(object) { standardGeneric("degrees_of_freedom")})
setGeneric("basis_matrix", 
    function(object) { standardGeneric("basis_matrix")})
setGeneric("spline_formula", 
    function(object) { standardGeneric("spline_formula")})
setGeneric("group_variable_name", 
    function(object) { standardGeneric("group_variable_name")})
setGeneric("group_variable_name<-", 
    function(object, value) { standardGeneric("group_variable_name<-")})
setGeneric("group_variable", 
    function(object) { standardGeneric("group_variable")})
setGeneric("group_variable<-", 
    function(object, value) { standardGeneric("group_variable<-")})
setGeneric("time_variable_name", 
    function(object) { standardGeneric("time_variable_name")})
setGeneric("time_variable_name<-", 
    function(object,  value) { standardGeneric("time_variable_name<-")})
setGeneric("time_variable", 
    function(object) { standardGeneric("time_variable")})
setGeneric("time_variable<-", 
    function(object,  value) { standardGeneric("time_variable<-")})
setGeneric("time_by_group_variable", 
    function(object) { standardGeneric("time_by_group_variable")})

#' @name Moanin-methods
#' @title Helper methods for the Moanin class
#'
#' @description This is a collection of helper methods for the Moanin
#'   class.
#' @inheritParams DE_timecourse
#' @param value replacement value
#' @param x \code{Moanin} object
#' @param ... arguments passed to subsetting
#' @examples 
#' # Load some data
#' data(exampleData)
#' moanin = create_moanin_model(data=testData,meta=testMeta)
#' group_variable_name(moanin)
#' time_variable_name(moanin)
#' @return \code{group_variable_name} and \code{time_variable_name} return the 
#' name of the column containing the variable. \code{group_variable} and 
#' \code{time_variable} return the  actual variable. 
#' @export
#' @aliases group_variable_name group_variable_name,Moanin-method
setMethod("group_variable_name","Moanin",function(object){
    object@group_variable_name
})
#' @rdname Moanin-methods
#' @aliases group_variable_name<- group_variable_name<-,Moanin-method
#' @export
setReplaceMethod("group_variable_name","Moanin",function(object,value){
    object@group_variable_name<-value
    return(object)
    
})
#' @rdname Moanin-methods
#' @aliases time_by_group_variable time_by_group_variable,Moanin-method
#' @export
setMethod("time_by_group_variable","Moanin",function(object){
    colData(object)$WeeklyGroup
})
#' @rdname Moanin-methods
#' @aliases group_variable group_variable,Moanin-method
#' @export
setMethod("group_variable","Moanin",function(object){
    gpVar<-group_variable_name(object)
    colData(object)[,gpVar]
})
#' @rdname Moanin-methods
#' @aliases group_variable<- group_variable<-,Moanin-method
#' @export
setReplaceMethod("group_variable","Moanin",function(object,value){
    gpVar<-group_variable_name(object)
    colData(object)[,gpVar]<-value
    return(object)
    
})
#' @rdname Moanin-methods
#' @aliases time_variable_name time_variable_name,Moanin-method
#' @export
setMethod("time_variable_name","Moanin",function(object){
    object@time_variable_name
})
#' @rdname Moanin-methods
#' @aliases time_variable_name<- time_variable_name<-,Moanin-method
#' @export
setReplaceMethod("time_variable_name","Moanin",function(object,value){
    object@time_variable_name<-value
    return(object)
    
})
#' @rdname Moanin-methods
#' @aliases time_variable time_variable,Moanin-method
#' @export
setMethod("time_variable","Moanin",function(object){
    tpVar<-time_variable_name(object)
    colData(object)[,tpVar]
})
#' @rdname Moanin-methods
#' @aliases time_variable<- time_variable_name<-,Moanin-method
#' @export
setReplaceMethod("time_variable","Moanin",function(object,value){
    tpVar<-time_variable_name(object)
    colData(object)[,tpVar]<-value
    return(object)
})
#' @rdname Moanin-methods
#' @aliases degrees_of_freedom degrees_of_freedom,Moanin-method
#' @export
setMethod("degrees_of_freedom","Moanin",function(object){
    object@degrees_of_freedom
})
#' @rdname Moanin-methods
#' @aliases basis_matrix basis_matrix,Moanin-method
#' @export
setMethod("basis_matrix","Moanin",function(object){
    object@basis_matrix
})
#' @rdname Moanin-methods
#' @aliases spline_formula spline_formula,Moanin-method
#' @export
setMethod("spline_formula","Moanin",function(object){
    object@spline_formula
})
#' @rdname Moanin-methods
#' @aliases show show,Moanin-method
#' @export
setMethod("show","Moanin",function(object){
        N<-ncol(object)
        cat("Moanin object on",N,
            "samples containing the following information:\n")
        cat(paste0("Group variable given by '",group_variable_name(object),
                   "' with the following levels:\n"))
        print(summary(group_variable(object)))
        cat(paste0("Time variable given by '",time_variable_name(object),"'\n"))
        cat("Basis matrix with",ncol(basis_matrix(object)),
            "basis_matrix functions\n")
        if(!is.null(spline_formula(object))){
            cat("Basis matrix was constructed with", 
                "the following spline_formula\n")
            #get rid of extra spaces:
            form<-gsub("\\s{2,}","",deparse(spline_formula(object))) 
            cat(paste(form,collapse="",sep=""),"\n")
        }
        else{
            if(is.null(degrees_of_freedom(object)))
                cat("Basis matrix was provided by user,",
                "spline_formula and degrees_of_freedom=NULL\n")
            else
                cat("Basis matrix and degrees of freedom provided by user,",
                    "equal to", degrees_of_freedom(object),"\n")
        }
        cat("\nInformation about the data (a SummarizedExperiment object):\n")
        show(as(object,"SummarizedExperiment"))
})


### For subsetting
#' @details Note that when subsetting the data, the dendrogram information and
#'   the co-clustering matrix are lost.
#' @aliases [,Moanin,ANY,ANY,ANY-method [,Moanin,ANY,character,ANY-method
#' @param i,j A vector of logical or integer subscripts, indicating the rows and
#'   columns to be subsetted for \code{i} and \code{j}, respectively.
#' @param drop A logical scalar that is ignored.
#' @rdname Moanin-methods
#' @export
setMethod(
    f = "[",
    signature = c("Moanin", "ANY", "character"),
    definition = function(x, i, j, ..., drop=TRUE) {
        j<-match(j, colnames(x))
        callGeneric()
        
    }
)
#' @rdname Moanin-methods
#' @export
setMethod(
    f = "[",
    signature = c("Moanin", "ANY", "logical"),
    definition = function(x, i, j, ..., drop=TRUE) {
        j<-which(j)
        callGeneric()
    }
)
#' @rdname Moanin-methods
#' @export
setMethod(
    f = "[",
    signature = c("Moanin", "ANY", "numeric"),
    definition = function(x, i, j, ..., drop=TRUE) {

        out<- new("Moanin",
                #have to explicitly give the inherintence... not great:
                as(selectMethod("[",
                    c("SummarizedExperiment","ANY","numeric"))(x,i,j),
                   "SummarizedExperiment"),
                basis_matrix=basis_matrix(x)[j, , drop=FALSE],
                spline_formula=spline_formula(x),
                degrees_of_freedom=degrees_of_freedom(x),
                group_variable_name=group_variable_name(x),
                time_variable_name=time_variable_name(x),
                log_transform=log_transform(x)
        )
        return(out)
    }
)

#' return log data if appropriate
#' @rdname Moanin-methods
#' @aliases log_transform log_transform,Moanin-method
#' @export
setMethod("log_transform","Moanin",function(object){
    object@log_transform
})

#' return log data if appropriate
#' @rdname Moanin-methods
#' @aliases get_log_data get_log_data,Moanin-method
#' @export
setMethod("get_log_data","Moanin",function(object){
    if(log_transform(object)) y<-log(assay(object)+1)
    else y<-assay(object)
    return(data.matrix(y))
})

#' Provides set of basis functions on either side of a time point, allowing for a discontinuity in the fitted functions
#' @param timepoints vector of numeric timepoints for which the splines basis will be evaluated
#' @param discont_point a single numeric value that represents where the discontinuity should be
#' @param knots passed to \code{ns} or \code{bs}. If not NULL, should give knots on either side of \code{discon_point} as single vector -- they will be separated in the call to \code{discon_point}
#' @param intercept Whether to include an intercept (vector of all 1s) for each side of the discontinuity. Note this is different than the argument \code{intercept} of either \code{bs} or \code{ns}, which is set to \code{FALSE}. 
#' @param type either "ns" or "bs" indicating which splines basis function should be used.
#' @param degree passed to \code{bs} (if applicable)
#' @param dfPre the df for the basis functions defined before the discontinuity point
#' @param dfPost the df for the basis functions defined after the discontinuity point
#' @examples
#' x<-seq(0,10,length=100)
#' basis<-discont_basis(x,discont_point=3, dfPre=3, dfPost=4, intercept=TRUE)
#' # Plot of the basis functions
#' par(mfrow=c(3,3))
#' for(i in 1:ncol(basis)){
#'    plot(x,basis[,i],type="l")
#'    abline(v=3,lty=2)
#' }
#' # Use it in a moanin_model object instead of ns/bs:
#' data(exampleData)
#' moanin <- create_moanin_model(data=testData, meta=testMeta,
#'     spline_formula=~Group:discont_basis(Timepoint,dfPre=3,
#'         dfPost=3,discont=20,intercept=TRUE)+0,
#'     degrees_of_freedom=6)
#' @export
discont_basis<-function(timepoints, discont_point, 
    knots=NULL, dfPre=NULL, dfPost=dfPre,degree=3, 
        intercept=TRUE,type=c("ns","bs")){

        # Useful for definition of intercept: https://cran.r-project.org/web/packages/crs/vignettes/spline_primer.pdf
        # Basically not the flat line intercept, but will create perfect multi-coliniearity if include it because for every observation will sum to 1 across the basis functions.
        
        
        #The df in command bs actually means the number of columns of the design matrix returned by bs.So if the intercept is not included in the design matrix (which is the default), then the df in command bs is equal to the real df minus 1....If intercept = TRUE, then we need m=df-2 knots, otherwise we need m=df-1 knots. http://publish.illinois.edu/liangf/files/2016/05/Note_splines.pdf

    if(!is.null(dfPre) & is.null(dfPost)) dfPost<-dfPre
    if(is.null(dfPre) & !is.null(dfPost)) dfPre<-dfPost        
    if(is.null(dfPre)& is.null(dfPost) & is.null(knots)) 
        stop("Must provide either dfPre and dfPost or knots")
    type<-match.arg(type)
    prePoints<-which(timepoints<=discont_point)                     
    postPoints<-which(timepoints>discont_point)    
    if(!is.null(knots)){
        knotsPre<-knots[knots<=discont_point]                 
        knotsPost<-knots[knots>discont_point]
    }
    else{
        knotsPre<-knotsPost<-NULL
    }
    bsfun<-switch(type,
        "ns"=function(...){splines::ns(...)},
        "bs"=function(...){splines::bs(degree=degree,...)}
        )  
    preBasis<-bsfun(x=timepoints[prePoints],
          knots=knotsPre,df=dfPre,
          intercept=FALSE)
    postBasis<-bsfun(x=timepoints[postPoints],
        knots=knotsPost,df=dfPost,
        intercept=FALSE)
                         
    if(intercept){
        preBasis<-cbind(rep(1,nrow(preBasis)),preBasis)
        postBasis<-cbind(rep(1,nrow(postBasis)),postBasis)
    }
    combinedBasis<-matrix(0,
        nrow=length(timepoints),
        ncol=ncol(preBasis)+ncol(postBasis))     
    
    preCols<-1:ncol(preBasis)
    postCols<-(ncol(preBasis)+1):(ncol(combinedBasis))
    combinedBasis[prePoints,preCols]<-preBasis
    combinedBasis[postPoints,postCols]<- postBasis
    return(combinedBasis)   
                      
}

library("moanin")
data("exampleData")

context("moanin::moanin_model.R")

test_that("moanin_model::create_moanin_model", {
    expect_silent(create_moanin_model(data=testData,meta=testMeta))
    formula = ~Group:splines::ns(Timepoint) + 0
    expect_silent(create_moanin_model(data=testData,meta=testMeta, 
                                      spline_formula=formula))
    basis = stats::model.matrix(formula, data=testMeta)
    expect_silent(create_moanin_model(data=testData,meta=testMeta, 
                                      basis_matrix=basis))

    expect_error(create_moanin_model(data=testData,meta=testMeta, 
                       basis_matrix=basis, spline_formula=formula))
})
test_that("moanin_model::discont_basis",{
    #test function
    x<-seq(0,10,length=100)
    basis<-discont_basis(x,discont_point=3, dfPre=3, dfPost=4, intercept=TRUE)
     
    # Use it in a moanin_model object instead of ns/bs:
    moanin <- create_moanin_model(data=testData, meta=testMeta,
        spline_formula=~Group:discont_basis(Timepoint,dfPre=3,
            dfPost=3,discont=20,intercept=TRUE)+0,
        degrees_of_freedom=6)
    
})

setGeneric("plot_splines_data", 
           function(object,data,...) { standardGeneric("plot_splines_data")})

#' Plotting splines
#'
#'@param data matrix containing the data to be plotted, where each row of the
#'  data provided will be plotted as a separate plot. If missing, will rely on
#'  data in \code{assay(object)}
#'@inheritParams DE_timecourse
#'@param colors vector, optional, default NULL. Vector of colors
#'@param smooth boolean, optional, default: FALSE. Whether to smooth the
#'  centroids or not.
#'@param mar vector of margins to set the space around each plot (see
#'  \code{\link{par}})
#'@param legend boolean whether to include a legend (default:TRUE)
#'@param legendArgs list of arguments to be passed to legend command (if
#'  \code{legend=TRUE})
#'@param simpleY boolean, if true, will plot all genes on same y-axis and
#'  minimize the annotation of the y axis to only label the axis in the exterior
#'  plots (the x-axis is always assumed to be the same across all plots and will
#'  always be simplified)
#'@param scale_centroid determines whether the centroid data given in
#'  \code{centroid} should be rescaled to match that of the data
#'  (\code{"toData"}), or the data scaled to match that of centroid
#'  (\code{"toCentroid"}), or simply plotted as is (\code{"none"}).
#'@param subset_conditions list if provided, only plots the subset of conditions
#'  provided. Else, plots all conditions
#'@param centroid numeric vector (or matrix of 1 row) with data to use to fit
#'  the splines. If \code{NULL}, the splines plotted will be from the data.
#'@param subset_data list if provided, only plots the subset of data (ie, the
#'  rows) provided. Can be any valid vector for subsetting a matrix. See
#'  details.
#'@param mfrow a vector of integers of length 2 defining the grid of plots to be
#'  created (see \code{\link{par}}). If missing, the function will set a value.
#'@param addToPlot A function that will be called after the plotting, allowing
#'  the user to add more to the plot.
#'@param xlab label for the x-axis
#'@param ylab label for the y-axis
#' @param xaxis Logical, whether to add x-axis labels to plot (if FALSE can be manually created by user with call to addToPlot)
#' @param yaxis Logical, whether to add y-axis labels to plot (if FALSE can be manually created by user with call to addToPlot)
#'@param ... arguments to be passed to the individual plot commands (Will be
#'  sent to all plot commands)
#' @details If \code{data} is NULL, the data plotted will be from
#'   \code{assay(object)}, after log-transformation if
#'   \code{log_transform(object)=TRUE}.
#' @details If \code{centroid} is missing, then splines will be estimated (per
#'   group) for the the data in \code{data} -- separately for each row of
#'   \code{data}. If \code{centroid} is provided, this data will be used to plot
#'   a spline function, and this same spline will be plotted for each row of
#'   \code{data}. This is useful, for example, in plotting cluster centroids
#'   over a series of genes.
#' @details If the user set \code{log_transform=TRUE} in the creation of the
#'   \code{Moanin} object, the data will be log transformed before plotting and
#'   calculating the spline fits.
#'@return This function creates a plot and does not return anything to the user.
#' @examples
#' # First, load some data and create a moanin model
#' data(exampleData)
#' moanin <- create_moanin_model(data=testData,meta=testMeta, 
#'    degrees_of_freedom=6)
#'
#' # The moanin model contains all the information for plotting purposes. The
#' # plot_splines_data will automatically fit the splines from the
#' # information contained in the moanin model
#' genes <- c("NM_001042489", "NM_008725")
#' plot_splines_data(moanin, subset_data=genes,
#' mfrow=c(2, 2))
#' # By default, same axis for all genes. Can change with 'simpleY=FALSE'
#' plot_splines_data(moanin, subset_data=genes,
#'    smooth=TRUE, mfrow=c(2,2), simpleY=FALSE)   
#'
#' # The splines can also be smoothed
#' plot_splines_data(moanin, subset_data=genes,
#'    smooth=TRUE, mfrow=c(2, 2))
#' # You can provide different data (on same subjects),
#' # instead of data in moanin object
#' # (in which case moanin just provides grouping information)
#' plot_splines_data(moanin, data=1/assay(moanin), subset_data=genes,
#'    smooth=TRUE, mfrow=c(2, 2))
#'    
#' # You can also provide data to use for fitting splines to argument  
#' # "centroid". This is helpful for overlaying centroids or predicted data
#' # Here we do a silly example, just to demonstrate syntax, 
#' # where we use the data from the first gene as our centroid to fit a
#' # spline estimate, but plot data from genes 3-4
#' plot_splines_data(moanin, centroid=assay(moanin[1,]), subset_data=3:4,
#'    smooth=TRUE, mfrow=c(2,2))
#' @export
#' @name plot_splines_data
#' @aliases plot_splines_data,Moanin,matrix-method
#' @importFrom graphics axis plot.new plot.window
#' @importFrom methods new
#' @importFrom viridis viridis
setMethod("plot_splines_data",c("Moanin","matrix"),
    function(object, data, colors=NULL, smooth=FALSE,
                legend=TRUE, legendArgs=NULL, 
                subset_conditions=NULL,
                subset_data=NULL,
                simpleY=TRUE, centroid=NULL,
                scale_centroid=c("toData","toCentroid","none"),
                mar=c(2.5, 2.5, 3.0, 1.0),
                mfrow=NULL, addToPlot=NULL, ylab="", xaxis=TRUE,
                yaxis=TRUE, xlab="Time",...){
    scale_centroid <- match.arg(scale_centroid)
    check_data_meta(data=data,object=object)
    if(!is.null(centroid)){
        check_data_meta(data=centroid,object=object)
        if(!is.null(dim(centroid)) && nrow(centroid)>1){
            stop("Centroid must be a single vector (or matrix of 1 row) to data, for fitting the functional form")
        } 
    }
    if(!is.null(subset_data) )  data <- data[subset_data, ,drop=FALSE]
    
    n_observations <- dim(data)[1]
    ### Work out the mfrow/number of plots and check makes sense
    n_plots <- if(legend) n_observations+1 else n_observations
    if(!is.null(mfrow)){
        if(length(mfrow) != 2){
            msg <- sprintf(
                paste0("Invalid value for argument mfrow.",
                       "Should be a vector of length 2.",
                       "A vector of length %s was provided.", length(mfrow)))
            stop(msg)
            
        }
        if(mfrow[1]*mfrow[2] < n_plots){
            msg <- sprintf(
                paste0(
                    "Invalid value for argument mfrow. Should result in ",
                    "grid for at least %s plots (including a plot for the ",
                    "legend, if legend=TRUE)"),
                n_plots) 
            stop(msg)
        }
    }else{
        if(n_plots <= 3){ 
            mfrow <- c(n_plots, 1)
        }else if(n_plots <= 6){
            mfrow <- c(ceiling(n_plots / 2), 2)
        }else if(n_plots <= 12){
            mfrow <- c(ceiling(n_plots / 3), 3)
        }else{
            nrow <- round(n_plots ** 0.5)
            ncol <- ceiling(n_plots / nrow)
            mfrow=c(ncol, nrow)
        }        
    }
    graphics::par(mfrow=mfrow, mar=mar)
    bottomPlots <- seq(to=n_observations, by=1, length=mfrow[2])
    sidePlots <- seq(from=1, to=n_observations, by=mfrow[2])
    
    
    ## For legend:
    if(is.null(colors)){
        groups <- levels(group_variable(object))
        colors <- viridis::viridis(length(groups))
        names(colors) <- groups
    }
    
    plot_names <- row.names(data)
    name <- NULL 
    if(!is.null(centroid)){
        if(scale_centroid=="toData"){
            centroid <- align_data_onto_centroid(data, centroid, 
                                                returnType="centroid")
        }
        if(scale_centroid=="toCentroid"){
            data <- align_data_onto_centroid(data, centroid, returnType="data")
        }
        if(scale_centroid %in% c("toCentroid","none")){
            #make it a matrix
            centroid <- matrix(centroid, nrow=nrow(data),ncol=length(centroid),
                              byrow=TRUE)
        }
    }
    
    ## Fix up the y-axis range
    if("ylim" %in% names(list(...))){
        ylim <- list(...)$ylim
    }
    else ylim <- NULL
    if(simpleY & is.null(ylim)){
        # use same y-axis range
        yrange <- range(data)
    }
    else{
        if(!is.null(ylim)) yrange <- ylim
        else yrange <- NULL
    }

    # Now plot the different data
    for(i in seq_len(n_observations)){
        if(!is.null(plot_names)){
            name <- plot_names[i]
        }
        xaxt<-if(!i %in% bottomPlots & xaxis) "n" else "s"
        if("xaxt" %in% names(list(...))) xaxt<-list(...)$xaxt
        yaxt<-if(!i %in% bottomPlots & yaxis) "n" else "s"
        if("yaxt" %in% names(list(...))) yaxt<-list(...)$yaxt
        plot_centroid_individual(
            data=data[i, ], centroid=if(is.null(centroid)) centroid else centroid[i,],
            object[i,], colors=colors,
            smooth=smooth,
            subset_conditions=subset_conditions,
            main=name, yrange=yrange,
            xaxt=xaxt,
            yaxt=yaxt, 
            xlab=xlab,ylab=ylab,...)
        
        if(is.function(addToPlot)){
            addToPlot()
        }
        
        if(!i %in% bottomPlots & xaxis){
            axis(1, labels=FALSE)
        }
        
        if(!i %in% sidePlots & simpleY & yaxis){
            axis(2, labels=FALSE)
        }
        
    }
    
    if(legend){
        plot.new()
        plot.window(
            xlim=c(0,1),
            ylim=c(0,1),
            bty="n",
            xaxt="n",
            yaxt="n")
        
        if(!is.null(subset_conditions)){
            colors <- colors[names(colors) %in% subset_conditions]
        }
        do.call(
            "legend",
            c(list(x="center",
                   legend=names(colors),
                   fill=colors, bty="n"),
              legendArgs))
    }
}
)

#' @aliases plot_splines_data,Moanin,numeric-method
#' @rdname plot_splines_data
#' @export
setMethod("plot_splines_data",c("Moanin","numeric"),
          function(object, data, ...){
              plot_splines_data(object,data=matrix(data,nrow=1),...)
          }
)
#' @aliases plot_splines_data,Moanin,data.frame-method
#' @rdname plot_splines_data
#' @export
setMethod("plot_splines_data",c("Moanin","data.frame"),
          function(object, data, ...){
              plot_splines_data(object,data=data.matrix(data),...)
          }
)

#' @aliases plot_splines_data,Moanin,DataFrame-method
#' @rdname plot_splines_data
#' @export
setMethod("plot_splines_data",c("Moanin","DataFrame"),
          function(object, data, ...){
              plot_splines_data(object,data=data.matrix(data),...)
          }
)
#' @aliases plot_splines_data,Moanin,missing-method
#' @rdname plot_splines_data
#' @export
setMethod("plot_splines_data",c("Moanin","missing"),
        function(object, data, ...){
            data<-get_log_data(object)
            plot_splines_data(object,data=data,...)
        }
)

# centroid and data are vectors
# moanin_model will separate them into groups...
plot_centroid_individual <- function(data, centroid, moanin_model,
                                    colors, smooth=FALSE, 
                                    subset_conditions=NULL,yrange=NULL,...){
    if(is.null(data)){
        data=as.vector(get_log_data(moanin_model))
    }
    if(is.null(centroid)) centroid<-data
    if(!inherits(moanin_model,"Moanin")) 
        stop("Internal coding error: expecting Moanin class")
    gpVar <- group_variable_name(moanin_model)
    tpVar <- time_variable_name(moanin_model)
    groups <- levels(group_variable(moanin_model))
    
    if(!is.null(subset_conditions)){
        if(!all(subset_conditions %in% groups)){
            msg <- paste0(
                "subset_conditions argument given by user does not match names",
                " of groups.")
            stop(msg)
        }else{
            groups <- subset_conditions
        }
    }
    
    xrange <- range(time_variable(moanin_model))
    if(inherits(centroid,"DataFrame")) centroid<-as.matrix(centroid)
    if(is.null(dim(centroid))) centroid <- t(as.matrix(centroid))
    if(is.null(yrange)){
        yrangeCentroid <- range(centroid[, group_variable(moanin_model) %in% groups])
        yrangeData <- range(data[group_variable(moanin_model) %in% groups])
        yrange <- range(c(yrangeCentroid,yrangeData))
    }
    graphics::plot(xrange, yrange, type="n", ...)
    if(smooth){
        # FIXME this is supposed to be on the fitted lines, but I'm not able
        # to get this to work fine in R.
        meta_prediction <- create_meta_prediction(moanin_model)
        centroid_fitted <- fit_predict_splines(
            centroid, moanin_model,
            meta_prediction=meta_prediction)
    }else{
        centroid_fitted <- fit_predict_splines(
            centroid, moanin_model)
    }
    
    
    # scatter points for values
    for(i in seq_along(groups)){
        group <- groups[i]
        color <- colors[group]
        
        # Start by individual points
        mask <- group_variable(moanin_model) == group
        time <- time_variable(moanin_model)[mask]
        indx <- order(time)
        graphics::lines(time[indx], data[mask][indx], type="p",
                        col=color, pch=16,
                        lwd=0)
        if(smooth){
            mask <- meta_prediction[,gpVar] == group
            time <- meta_prediction[,tpVar][mask]
            indx <- order(time)
        }
        graphics::lines(time[indx], centroid_fitted[mask][indx], type="l",
                        col=color, lwd=1)
        
    }
}

#' Find enriched GO terms
#'
#' @param assignments boolean named vector determining the gene subset to be
#'   tested for enrichment of GO terms. The names of the vector should be the
#'   gene names. Elements with TRUE will consist of the gene cluster.
#' @param gene_id_to_go List giving the Gene ID to GO object required for topGO
#'   (see \code{\link[topGO]{topGOdata-class}}). 
#'   \code{create_go_term_mapping} can construct such a list from a
#'   data-frame.
#'@param ontology string, optional, default: BP. specficies which ontology to
#'  use (passed to \code{ontology} argument in creating a new \code{topGOdata}
#'  object). Can be 'BP', 'CC', or 'NF'. See
#'  \code{\link[topGO]{topGOdata-class}}.
#'@param weighted boolean, optional, default: FALSE. Whether to use the
#'  weighted algorithm or not in \code{\link[topGO]{runTest}}.
#'@param node_size integer, optional, default: 10. Consider only GO terms with
#'  node_size number of genes, passed to \code{nodeSize} argument of
#'  \code{\link[topGO]{topGOdata-class}}
#'@details \code{find_enriched_go_terms} is a wrapper for running a GO
#'  enrichment analysis via the package \code{topGO}. This function creates a
#'  \code{\link[topGO]{topGOdata-class}} object, runs the function
#'  \code{\link[topGO]{runTest}} to test for enrichment using the
#'  \code{statistic="fisher"} option, and then runs
#'  \code{\link[topGO]{GenTable}}. This function then does some post-processing
#'  of the results, returning only GO terms that satisfy:
#'  \enumerate{
#'  \item{BH adjusted p-values less than 0.05 using
#'  \code{\link[stats]{p.adjust}}}
#'  \item{GO terms are \emph{enriched}, i.e. the number of genes from the GO 
#'  term found in the subset is greater than expected}
#'  }
#' @return Returns results in the format of \code{\link[topGO]{GenTable}}.
#' @aliases create_go_term_mapping
#' @examples 
#' data(exampleData)
#' head(testGenesGO) #gives the mapping of genes to GO
#' geneId2Go <- create_go_term_mapping(testGenesGO)
#' #create fake assignment of genes to group based on TRUE/FALSE values
#' inGroup=rep(FALSE,nrow(testData))
#' inGroup[1:10]=TRUE
#' names(inGroup) <- row.names(testData)
#' find_enriched_go_terms(inGroup, geneId2Go)
#' @seealso \code{\link{create_go_term_mapping}}, \code{\link[KEGGprofile]{find_enriched_pathway}}, \code{\link[topGO]{GenTable}}, \code{\link[topGO]{runTest}}, \code{\link[topGO]{topGOdata-class}}, \code{\link[stats]{p.adjust}} 
#' @export
#' @importFrom topGO annFUN.gene2GO runTest GenTable
find_enriched_go_terms <- function(assignments, gene_id_to_go,
                                  ontology="BP", 
                                  weighted=FALSE,
                                  node_size=10){
    if(is.null(names(assignments))){
        stop("assignments must be a named vector,",
             "where names match names of gene_id_to_go ")
    }
    gene_names <- names(assignments)
    assignments <- as.numeric(assignments)
    names(assignments) <- gene_names
    if(!(ontology %in% c("BP", "CC", "NF"))){
        error_message <- paste(
            "moanin::find_enriched_go_terms: Ontology should be 'BP', 'CC',",
            "or 'NF'. Ontology provided is",
            ontology)
        stop(error_message)
    }
    
    # function to specify which genes are interesting based on the gene scores.
    # It should be present iff the allGenes object is of type numeric.
    getTopDiffGenes <- function(data, cutOff=NULL){
        return(data < 0.5)
    }
    
    GOdata <- new("topGOdata", ontology=ontology, allGenes=assignments,
                 nodeSize=node_size,
                 geneSel=getTopDiffGenes,
                 annot=topGO::annFUN.gene2GO, gene2GO=gene_id_to_go)
    
    if(weighted){
        resultFisher <- topGO::runTest(GOdata, algorithm="classic", 
                                      statistic="fisher")
    }else{
        resultFisher <- topGO::runTest(GOdata, algorithm="weight", 
                                      statistic="fisher")
    }
    n_nodes <- length(resultFisher@score)
    
    allRes <- topGO::GenTable(
        GOdata, 
        resultFisher=resultFisher,
        orderBy="resultFisher", ranksOf="resultFisher",
        topNodes=n_nodes)
    
    # P-value correct
    allRes[, "resultFisher_padj"] <- stats::p.adjust(allRes$resultFisher, 
                                                    method="BH")
    wh <- which(allRes[, "resultFisher"] <= 0.05)
    
    allRes <- allRes[wh,]
    wh <- which(apply(allRes[, c("Significant", "Expected")],
                     1, function(x){x["Significant"] > x["Expected"]}))
    allRes <- allRes[wh,]
    
    return(allRes)
}


#' Create the Gene to GO Term mapping
#'
#' @param genes dataframe, with two required columns. The first gives the gene
#'   names, with column name by the argument \code{gene_col}. The other column
#'   must be named "go_id" and give the genes GO id. Genes will have multiple GO
#'   id that they map to, and each go mapping of a gene is a separate row. Thus
#'   genes will be in multiple rows of the input.
#' @param gene_col the name of the column of the \code{genes} data frame that
#'   contains the correct gene reference. By default, is "refseq_mrna".
#' @return \code{create_go_term_mapping} returns a list giving the gene to GO id
#'   in the format required by \code{\link[topGO]{topGOdata-class}}.
#' @rdname find_enriched_go_terms
#' @export
create_go_term_mapping <- function(genes, gene_col="refseq_mrna"){
    gene_id_go_mapping <- NULL
    gene_names <- unique(genes[, "refseq_mrna"])
    
    i <- 1
    for(gene in gene_names){
        go_terms <- genes[genes[, gene_col] == gene, "go_id"]
        if(length(go_terms) != 0){
            gene_id_go_mapping$gene <- go_terms
            names(gene_id_go_mapping)[i] <- gene
            i <- i + 1
        }
    }
    return(gene_id_go_mapping)
}

#setOldClass("formula", S4Class="formula")
setClassUnion("numericOrNULL",members=c("numeric", "NULL"))
setClassUnion("formulaOrNULL",members=c("formula", "NULL"))

#' @title Class Moanin
#'
#' @description \code{Moanin} is a class that extends
#'   \code{SummarizedExperiment} and is used to store the additional spline
#'   basis and meta data for timecourse analysis.
#'
#' @docType class
#' @aliases Moanin Moanin-class
#' @description In addition to the slots of the \code{SummarizedExperiment}
#' class, the \code{Moanin} object has the additional slots described
#' in the Slots section.
#'
#' @description There are several methods implemented for this class. The most
#'   important methods have their own help page. Simple helper methods are
#'   described in the Methods section below. For a comprehensive list of methods
#'   specific to this class see the Reference Manual.
#' @slot time_variable_name character value giving the column in \code{colData}
#'   that defines the time variable (must be of class \code{numeric})
#' @slot group_variable_name character value giving the column in \code{colData}
#'   that defines the grouping variable (must be of class \code{factor})
#' @slot basis_matrix A basis matrix, where each row corresponds to the
#'   evaluation of a sample on the basis function (thus one column for each
#'   basis function).
#' @slot spline_formula a formula. The formula used in creating the basis matrix
#' @slot degrees_of_freedom a numeric integer. Number of degrees of freedom used
#'   in creating basis matrix. If NULL, degrees of freedom is not known (usually
#'   if user provided basis without degrees of freedom)
#' @slot log_transform logical, whether to log-transform the data for certain
#'   methods
#' @name Moanin-class
#' @aliases Moanin
#' @rdname Moanin-class
#' @import SummarizedExperiment
#' @import methods
#' @export
#'
setClass(
    Class = "Moanin",
    contains = "SummarizedExperiment",
    slots = list(
        spline_formula="formulaOrNULL", 
        basis_matrix="matrix",
        group_variable_name="character",
        time_variable_name="character",
        degrees_of_freedom="numericOrNULL",
        log_transform="logical"
    )
)

setValidity("Moanin", function(object) {
    if(! group_variable_name(object) %in% colnames(colData(object))) 
        return(paste("group_variable_name slot must match the name",
            "of one of the columns of colData"))
    else{
        if(!is.factor(group_variable(object))) 
            return(paste(group_variable_name(object),
                "is not a numeric column in the colData of the object"))
    }
    if(! time_variable_name(object) %in% colnames(colData(object))) 
        return(paste("time_variable_name slot must match the", 
            "name of one of the columns of colData"))
    else{
        if(!is.numeric(time_variable(object))) 
            return(paste(time_variable_name(object),
                "is not a numeric column in the colData of the object"))
    }
    if(nrow(basis_matrix(object)) != ncol(object)) 
        return(paste("number of rows of the basis_matrix doesn't match",
            "number of samples of object"))

    return(TRUE)
})

setGeneric(
    name = "create_moanin_model",
    def = function(data,  ...) {
        standardGeneric("create_moanin_model")
    }
)
#'Create a Moanin object
#'@description The constructor \code{create_moanin_model} creates an object of
#'  the class \code{Moanin}.
#'
#'@param data The input data. Can be a \code{\link{SummarizedExperiment}} class,
#'  or matrix/data.frame. If the input data is a \code{matrix} or
#'  \code{data.frame}, then the user must also provide input to the \code{meta}
#'  argument, which will be transformed into \code{colData} of the resulting
#'  \code{Moanin} object
#'@param meta Meta data on the samples (columns) of the \code{data} argument.
#'  Must be given f input \code{data} is a \code{matrix} or \code{data.frame}.
#'  If input is \code{SummarizedExperiment}, this argument is ignored.
#'@param group_variable_name A character value giving the column that
#'  corresponds to the grouping variable to test for DE. By default "Group"
#'@param time_variable_name A character value giving the column that corresponds
#'  to the time variable. By default "Timepoint".
#'@param spline_formula formula object, optional, default: NUlL. Used to
#'  construct splines from the data in \code{meta}. See details.
#'@param basis_matrix matrix, optional, default: NULL. A basis matrix, where
#'  each row corresponds to the evaluation of a sample on the basis function
#'  (thus one column for each basis function).
#'@param degrees_of_freedom int, optional. Number of degrees of freedom to use
#'  if neither the basis_matrix nor the spline_formula is provided. If not
#'  provided by the user, internally will be set to 4
#'@param log_transform whether the data should be log-transformed by certain
#'  methods (see \code{\link{splines_kmeans}})
#'@param drop_levels Logical, whether to perform \code{\link{droplevels}} on the
#'  grouping variable (i.e. remove empty levels)
#'@param ... arguments passed from methods to the \code{SummarizedExperiment}
#'  method.
#'@details If neither \code{spline_formula} nor \code{basis_matrix} is given,
#'  then by default, the function will create a basis matrix based on the
#'  formula: \preformatted{spline_formula = ~Group:ns(Timepoint, df=4) + Group +
#'  0}
#'@details Note that the meta data will have levels dropped (via 
#'\code{droplevels}). 
#'@details Input to \code{data} that is given as a class \code{DataFrame} or
#'  \code{data.frame} will be converted to class \code{matrix}. The reason
#'  for this is that use of a \code{data.frame} creates errors in taking duplicate rows/columns of \code{SummarizedExperiment}, as in bootstrapping. 
#'  Users who absolutely want the object to hold a
#'  object that is not a matrix can construct a \code{SummarizedExperiment} object
#'  (which will not convert the input into a \code{matrix}), and
#'  use this as input to \code{create_moanin_model}.
#'@return An object of class \code{Moanin}
#' @examples
#' # Load some data
#' data(exampleData)
#'
#' # Use the default options
#' moanin = create_moanin_model(data=testData,meta=testMeta)
#' moanin
#'
#' # Change the number of degrees of freedom
#' moanin = create_moanin_model(data=testData,meta=testMeta, 
#'    degrees_of_freedom=6)
#' moanin
#' @export
#' @importFrom splines ns
#' @importFrom stats as.formula
#' @rdname Moanin-class
#' @aliases create_moanin_model
#' @aliases Moanin
#' @importClassesFrom S4Vectors DataFrame
#' @export
setMethod(
    f = "create_moanin_model",
    signature = signature("DataFrame"),
    definition = function(data, meta, ...){
        create_moanin_model(data.matrix(data),...)
    })
#' @rdname Moanin-class
#' @export
setMethod(
    f = "create_moanin_model",
    signature = signature("data.frame"),
    definition = function(data, ...){
        create_moanin_model(data.matrix(data), ...)
    })
#' @rdname Moanin-class
#' @export
setMethod(
    f = "create_moanin_model",
    signature = signature("matrix"),
    definition = function(data,  meta,...){
        if(missing(meta)) stop("Must provide argument meta if input to data is",
                "matrix-like")
        create_moanin_model(SummarizedExperiment(data, colData=meta),...)
    })
#' @rdname Moanin-class
#' @export
setMethod(
    f = "create_moanin_model",
    signature = signature("SummarizedExperiment"),
    definition = function(data, spline_formula=NULL, basis_matrix=NULL,
                group_variable_name="Group",time_variable_name="Timepoint",
                degrees_of_freedom=NULL,log_transform=FALSE,drop_levels=TRUE){

    if(!is.null(basis_matrix) & !is.null(spline_formula)){
        msg = paste("both basis_matrix and spline_formula ",
                    "are provided by the user. Please provide one or ",
                    "the other", sep="")
        stop(msg)
    }
    if(!("WeeklyGroup" %in% colnames(colData(data)))){
        colData(data)$WeeklyGroup = as.factor(
            make.names(colData(data)[,group_variable_name]:as.factor(
                colData(data)[,time_variable_name])))
    }
    # Must be done *before* build basis
    if(drop_levels){
        colData(data)[,group_variable_name]<-
            droplevels(colData(data)[,group_variable_name])
    }        
    if(is.null(basis_matrix)){
        if(is.null(spline_formula)){
            if(is.null(degrees_of_freedom)){
                degrees_of_freedom = 4
            }
            formulaText<-paste0("~",group_variable_name," + ",
                                group_variable_name,
                                ":splines::ns(",time_variable_name,
                                ",df=",degrees_of_freedom,") + 0")
            spline_formula = stats::as.formula(formulaText)
        }
        basis_matrix = stats::model.matrix(spline_formula, data=colData(data))
    }else{
        basis_matrix = as.matrix(basis_matrix)
    }
    
    splines_model = new("Moanin", data,
                        degrees_of_freedom=degrees_of_freedom,
                        basis_matrix=basis_matrix,
                        spline_formula=spline_formula,
                        time_variable_name=time_variable_name,
                        group_variable_name=group_variable_name,
                        log_transform=log_transform
                        )
    # Just create this one.
    
    return(splines_model)
    }
)

setGeneric("estimate_log_fold_change", 
           function(object,...) { standardGeneric("estimate_log_fold_change")})

#' Estimates log fold change
#'
#' @inheritParams DE_timecourse
#' @param contrasts The contrasts to consider
#' @param method method for calculating the log-fold change. See details. 
#' @details The following methods exist for calculating the log-fold change
#'   between conditions over time (default is "timecourse"):
#' \itemize{
#' \item{\code{timely}}{The log-fold change for each individual timepoint
#' (\eqn{lfc(t)})}
#' \item{\code{timecourse}}{The average absolute per-week fold-change,
#' multiplied by the sign of the average per-week fold-change.}
#' \item{\code{sum}}{Sum of per-week log fold change, over all timepoints}
#' \item{\code{max}}{Max of per-week log fold change, over all timepoints}
#' \item{\code{abs_sum}}{Sum of the absolute value of the per-week log fold
#' change, over all timepoints}
#' \item{\code{abs_squared_sum}}{Sum of the square value of the per-week log
#' fold change, over all timepoint}
#' \item{\code{min}}{Min of per-week log fold change, over all timepoints}
#' }
#' @details If the user set \code{log_transform=TRUE} in the creation of the
#'   \code{Moanin} object, the data will be log transformed before calculating
#'   the fold-change.
#' @return A data.frame giving the estimated log-fold change for each gene
#'   (row). For all methods except for "timely", the data frame will consist of
#'   one column for each value of the argument \code{contrasts}. For "timely"
#'   there will be one column for each timepoint and contrast combination.
#' @name estimate_log_fold_change
#' @aliases estimate_log_fold_change,Moanin-method
#' @examples 
#' data(exampleData)
#' moanin <- create_moanin_model(data=testData,meta=testMeta)
#' estsTimely <- estimate_log_fold_change(moanin, 
#'    contrasts=c("K-C"), method="timely")
#' head(estsTimely)
#' estsTimecourse <- estimate_log_fold_change(moanin, 
#'    contrasts=c("K-C"),method="timecourse")
#' head(estsTimecourse)
#' @export
setMethod("estimate_log_fold_change", "Moanin",
          function(object, 
        contrasts, 
        method=c("timecourse", "sum", "max", "timely", 
                 "abs_sum", "abs_squared_sum", "min")){

    tpVar <- time_variable_name(object)
    gpVar <- group_variable_name(object)
    method <- match.arg(method)
    contrasts <- is_contrasts(contrasts, object)
    
    if(method == "timely"){
        log_fold_changes <- lfc_per_time(object, contrasts)
    }else if(method %in% c("sum", "max", "min", "abs_sum", "abs_squared_sum",
                           "timecourse", "epicon")){
        timely_lfc <- lfc_per_time(object, contrasts)
        timely_lfc_meta <- reconstruct_meta_from_lfc(timely_lfc, split_char=":",
                    group_variable_name=gpVar,time_variable_name=tpVar)
        log_fold_changes <- data.frame(row.names=row.names(object))
        for(contrast in colnames(contrasts)){
            mask <- (timely_lfc_meta[,gpVar] == contrast) & 
                !is.na(matrixStats::colSums2(timely_lfc))
            if(method == "max"){
                log_fold_changes[, contrast] <- matrixStats::rowMaxs(abs(timely_lfc[, mask]))
            } else if(method == "min") {
                log_fold_changes[, contrast] <- matrixStats::rowMins(abs(timely_lfc[, mask])) 
            }else if(method == "abs_sum"){
                log_fold_changes[, contrast] <- matrixStats::rowSums2(abs(timely_lfc[, mask]))
            }else if(method == "abs_squared_sum"){
                log_fold_changes[, contrast] <- matrixStats::rowSums2(timely_lfc[, mask]**2)
            }else if(method == "epicon" || method == "timecourse"){
                log_fold_changes[, contrast] <- (
                    matrixStats::rowMeans2(abs(timely_lfc[, mask])) * 
                        sign(matrixStats::rowSums2(timely_lfc[, mask])))
            }else if(method == "sum"){
                log_fold_changes[, contrast] <- matrixStats::rowSums2(timely_lfc[, mask])
            }
        }
        
    }
    
    return(log_fold_changes)
    
}
)

# XXX helper function to reconstruct metadat from adat
reconstruct_meta_from_lfc <- function(data_per_time, group_variable_name,
                                     time_variable_name,split_char="."){
    meta_per_time <- t(
        as.data.frame(strsplit(colnames(data_per_time), split_char, 
                               fixed=TRUE)))
    row.names(meta_per_time) <- colnames(data_per_time)
    colnames(meta_per_time) <- c(group_variable_name, time_variable_name)
    meta_per_time <- as.data.frame(meta_per_time)
    meta_per_time[, time_variable_name] <- 
        as.numeric(meta_per_time[, time_variable_name])
    return(meta_per_time)
}


lfc_per_time <- function(object, contrasts){
    tpVar <- time_variable_name(object)
    gpVar <- group_variable_name(object)
    group_variable(object) <- as.factor(group_variable(object))
    averaged_data <- average_replicates(object)
    averaged_meta <- reconstruct_meta_from_lfc(averaged_data, split_char=":",
                                group_variable_name=gpVar,
                                time_variable_name=tpVar)
    
    averaged_meta[,tpVar] <- as.factor(averaged_meta[,tpVar])
    sample_coefficients <- vapply(
        averaged_meta[,gpVar], 
        function(x) return(contrasts[x, ]),
        numeric(ncol(contrasts)))

    if(is.null(dim(sample_coefficients))){
        sample_coefficients <- as.matrix(sample_coefficients)
    }else{
        sample_coefficients <- t(sample_coefficients)
    }

    colnames(sample_coefficients) <- colnames(contrasts)
    
    log_fold_changes <- matrix(NA, dim(object)[1],
                     dim(contrasts)[2]*length(unique(time_variable(object))))
    row.names(log_fold_changes) <- row.names(object)
    colnames(log_fold_changes) <- vapply(
        colnames(sample_coefficients),
        FUN=function(x){vapply(unique(averaged_meta[,tpVar]), 
                               FUN=function(t){paste0(x, ":", t)},
                               character(1))},
        character(length(unique(averaged_meta[, tpVar]))))
    
    for(column in colnames(sample_coefficients)){
        
        sample_coefficient <- as.vector(unlist(sample_coefficients[, column]))
        coef_data <- t(t(averaged_data) * sample_coefficient)
        for(timepoint in unique(averaged_meta[,tpVar])){
            mask <- averaged_meta[,tpVar] == timepoint
            if(sum(mask) != dim(contrasts)[1]){
                next
            }
            colname <- paste0(column, ":", as.character(timepoint))
            log_fold_changes[, colname] <- matrixStats::rowSums2(coef_data[, mask])
        }
    }
    return(log_fold_changes)
}


average_replicates <- function(object){
    timepoint_group <- 
        droplevels(group_variable(object):as.factor(time_variable(object)))
    all_levels <- levels(timepoint_group)
    
    # selecting certain columns sometimes returns vectors and sometimes 
    # matrices depending on whether the user wants a single column or several
    # columns. rowMeans requires a matrix. Thus, here's a small implementation
    # that will work in all cases.
    .row_means <- function(data){
        if(is.vector(data)){
            return(data)
        }else{
            return(matrixStats::rowMeans2(data))
        }
    }
    y <- get_log_data(object)
    if(inherits(y, "DataFrame")){
        y <- data.matrix(y)
    }
    replicate_averaged <- vapply(
        all_levels,
        function(m){.row_means(y[, timepoint_group==m])},
        numeric(nrow(y)))
    return(replicate_averaged)
}

    

setGeneric("DE_timecourse", 
           function(object,...) { standardGeneric("DE_timecourse")})

# This file contains EDGE code adapted to work with limma contrasts

center_data <- function(y, ng_labels){
    for(g in levels(ng_labels)){
        whKeep <- which(ng_labels == g)
        sub_mean <- matrixStats::rowMeans2(y[, whKeep])
        y[, whKeep] <- y[, whKeep] - sub_mean
    }
    return(y)
}

# updated function to compute beta under the null. 
compute_beta_null<-function(moanin_model,contrast_matrix,beta){
    Xmat<-basis_matrix(moanin_model) #n x p
    if(ncol(contrast_matrix)!=ncol(Xmat)) stop("invalid contrast_matrix provided, wrong dimensions (number of columns)")
    
    invdesign<-MASS::ginv(t(Xmat)%*%Xmat)
    invCdesign<-MASS::ginv(contrast_matrix %*%invdesign %*% t(contrast_matrix))
    Wmat<-invdesign%*%t(contrast_matrix)%*%invCdesign%*%contrast_matrix
    # # The full upper matrix (for checking)
    # Rmat<-invdesign-Wmat%*%invdesign
    
    return(beta-beta%*%t(Wmat))
    
}
#Expands contrasts to a matrix of contrasts, one for each basis vector/variable. rows are contrasts, columns are the number of total variables. 
#' @keywords internal
#' @rdname internal
expand_contrast<-function(moanin_model,contrast_vector){
    design_matrix<-basis_matrix(moanin_model)
    gpname<-group_variable_name(moanin_model)
    if(length(grep(gpname, colnames(design_matrix)))==0) 
        stop("basis_matrix does not have column names that include" +
            "the group_variable_name, cannot return full contrast matrix")
    if(is.null(names(contrast_vector))) stop("Error, no names")
    levs<-names(contrast_vector)
    glevs<-paste0(gpname,levs)
    #Check that replicated the same:
    m<-lapply(glevs,grep,colnames(design_matrix))
    nvar_per_level<-vapply(m,FUN=length,numeric(1))
    if(!all(nvar_per_level==nvar_per_level[1]))
        stop("There are not equal numbers of variables (basis functions) per level of grouping factor")
    nvar_per_level<-unique(nvar_per_level)
    
    #find the variables for each level (`vars`)
    full_vars<-colnames(design_matrix)[m[[1]]]
    vars<-gsub(glevs[1],"",full_vars)
    # check they are all there
    combos<-as.vector(outer(glevs,vars,FUN=paste0))
    if(!all(combos %in% colnames(design_matrix)))
        stop("Not all the same variables are crossed with the group variable")
    
    contrast_full<-matrix(0,
        nrow=length(vars),
        ncol=ncol(design_matrix))
    colnames(contrast_full)<-colnames(design_matrix)
    for(ii in seq_along(vars)){
        varnames<-paste0(glevs,vars[ii])
        m<-match(varnames,colnames(design_matrix))
        if(any(is.na(m))) stop("coding error in matching to design matrix")
        contrast_full[ii,m]<-contrast_vector
    }
    return(contrast_full)
}


calculateStat<-function(resNull,resFull,type=c("lrt","ftest")){
    type<-match.arg(type)
    #resNull and resFull are G x n matrices of residuals
    if(!all(dim(resNull)==dim(resFull))) 
        stop("resNull and resFull must be of equal dimensions")
    n<-ncol(resNull)
    ss0 <- matrixStats::rowSums2(resNull^2) #RSSNull
    ss1 <- matrixStats::rowSums2(resFull^2) #RSSFull
    if(type=="lrt") 
        return(n*(log(ss0)-log(ss1)))
    # Note that F-statistic returned below is intentionally missing the 
    # df terms, which are done later when calculating the p-values 
    # (this is for simplicity so don't have to pass df to this function.)
    if(type=="ftest") 
        return((ss0 - ss1)/(ss1))

}


compute_pvalue <- function(basis, y, beta, beta_null, 
                          degrees_of_freedom=NULL,
                          statistics=c("lrt","ftest"),
                          weights=NULL){
    statistics<-match.arg(statistics)
    if(inherits(y,"DataFrame")){
        y <- data.matrix(y)
    }
    #Creates G x n matrices, with fits on the rows
    fitFull <- beta %*% t(basis)    
    fitNull <- beta_null %*% t(basis)
    
    if(!is.null(weights)){
        resNull <- weights^(1/2) * (y - fitNull)
        resFull <- weights^(1/2) * (y - fitFull)
    }else{
        resNull <- y - fitNull
        resFull <- y - fitFull
    }
    n_variables<-ncol(basis) #p
    n_samples <- nrow(basis) 
    stat<-calculateStat(resNull,resFull,type=statistics)
    if(statistics == "ftest"){
        df2 <- n_samples - n_variables
        df1 <- degrees_of_freedom
        pval <- stats::pf(stat * df2 / df1, df1=df1, df2=df2, lower.tail=FALSE)
    }else{
        pval <- stats::pchisq(stat, df=degrees_of_freedom, lower.tail=FALSE)
    }
    return(cbind(pval,stat))
}


#' Run spline models and test for DE of contrasts.
#' 
#' @param object An object of class \code{\link{Moanin}}, an object containing
#'   all related information for time course data and the splines model that
#'   will be used (if applicable). See \code{\link{create_moanin_model}} for
#'   more details.
#'@param contrasts Contrasts, either provided as a vector of strings, or a
#'  matrix of contrasts coefficients obtained using
#'  \code{\link[limma]{makeContrasts}} from the package \code{limma}. If given
#'  as a character string, will be passed to \code{\link[limma]{makeContrasts}}
#'  to be converted into such a matrix.
#' @param statistic Which test statistic to use, a likelihood ratio statistic or
#'  a F-test.
#' @param center boolean, whether to center the data matrix
#' @param use_voom_weights boolean, optional, default: TRUE. 
#'  Whether to use voom weights. See details.
#' @details The implementation of the spline fit and the calculation of p-values
#'   was based on code from \code{\link[edge]{edge}}, and expanded to enable
#'   handling of comparisons of groups via contrasts. The code assumes that the \code{Moanin} object was created via either a formula or a basis where a different spline was fit for each \code{group_variable} and thus the contrasts are comparisons of those spline fits. If the \code{Moanin} object was created via user-provided basis matrix or formula, then the user should take a great deal of caution in using this code, as the degrees of freedom for the tests of significance cannot be verified to be correct. 
#' @seealso
#'   \code{\link[limma]{makeContrasts}}, \code{\link{create_moanin_model}},
#'   \code{\link{DE_timepoints}}, \code{\link[edge]{edge}}
#' @details If \code{use_voom_weights=TRUE}, then before fitting splines to each gene,
#' voom weights are calculated from \code{assay(object)}:
#' \preformatted{
#'    y <- edgeR::DGEList(counts=assay(object))
#'    y <- edgeR::calcNormFactors(y, method="upperquartile")
#'    v <- limma::voom(y, design, plot=FALSE)
#'    weights <- v$weights
#' }
#' The design matrix for the voom weights is based on the formula
#' \code{~Group + Timepoint +0}
#' where Group and Timepoint are replaced with the user-defined values where appropriate. 
#' These weights are given to the \code{lm.fit} which fits the spline coefficients.
#' This workflow assumes that the input to the \code{Moanin} object were counts.
#' @details If the user set \code{log_transform=TRUE} in the creation of the
#'   \code{Moanin} object, the splines will be fit to the log of the input data,
#'   and not directly to the input data. This is independent of whether the user
#'   chooses \code{use_voom_weights}.
#' @return A \code{data.frame} with two columns for each of the contrasts given
#'   in \code{contrasts}, corresponding to the raw p-value of the contrast for
#'   that gene (\code{_pval}) and the adjusted p-value (\code{_qval}). The
#'   adjusted p-values are FDR-adjusted based on the Benjamini-Hochberg method,
#'   as implemented in \code{\link[stats]{p.adjust}}. The adjustment is done
#'   across all p-values for all contrasts calculated.
#' @examples 
#' data(exampleData)
#' moanin <- create_moanin_model(data=testData, meta=testMeta)
#' deTimecourse=DE_timecourse(moanin, 
#'    contrasts="K-C", use_voom_weights=FALSE)
#' head(deTimecourse)
#' @name DE_timecourse
#' @aliases DE_timecourse,Moanin-method
#' @export
setMethod("DE_timecourse","Moanin",
         function(object,
                  contrasts,
                  center=FALSE, statistic=c("ftest","lrt"),
                  use_voom_weights=TRUE){
    statistic<-match.arg(statistic)            
    basis <- basis_matrix(object)    
    ng_labels <- group_variable(object)
    ng <- nlevels(ng_labels)

    # Will make this a matrix, if not already one, 
    # using limma's makeContrasts. 
    # So returns a matrix, column for each contrast
    contrasts <- is_contrasts(contrasts, object)
    if(dim(contrasts)[1] != ng){
        stop("The contrast coef vector should be of the same size" +
                 " as the number of groups")
    }


    if(use_voom_weights){
        formulaText<-paste0("~",group_variable_name(object)," + ",
                            time_variable_name(object)," + 0")
        voomFormula = stats::as.formula(formulaText)
        
        design <- stats::model.matrix(voomFormula,
            data=colData(object))
        
        y <- edgeR::DGEList(counts=assay(object))
        y <- edgeR::calcNormFactors(y, method="upperquartile")
        v <- limma::voom(y, design, plot=FALSE)
        weights <- v$weights
    }else{
        weights <- NULL
    }
    
    y<-get_log_data(object)
    
    if(center){
        y <- center_data(y)
        basis <- t(center_data(t(basis)))
    }
    
    beta <- fit_splines(data=y, design_matrix=basis_matrix(object), weights=weights)

    results <- data.frame(row.names=row.names(object))
    for(col in seq_len(ncol(contrasts))){
        contrast <- contrasts[, col]
        
        # Create the name of the column
        contrast_name <- colnames(contrasts)[col]
        contrast_name <- gsub(" ", "", contrast_name, fixed=TRUE)
                
        # Right now uses `expand_contrasts` to create full contrast matrix (one per basis function), sort of a hack.
        # But could let user provide it...
        contrast_matrix<-expand_contrast(object,contrast)
        beta_null<-compute_beta_null(object,
            contrast_matrix=contrast_matrix,beta)
        
        pval <- compute_pvalue(basis, y, beta, beta_null, 
                              weights=weights,statistics=statistic,
                              degrees_of_freedom=nrow(contrast_matrix))
        
        colname_qval <- paste(contrast_name, "_qval", sep="")
        colname_pval <- paste(contrast_name, "_pval", sep="")
        colname_stat <- paste(contrast_name, "_stat", sep="")
        
        results[colname_stat] <- pval[,2]
        results[colname_pval] <- pval[,1]
        qval <- stats::p.adjust(pval[,1], method="BH")
        results[colname_qval] <- qval
    }
    
    return(results)
})

setGeneric("DE_timepoints", 
           function(object,...) { standardGeneric("DE_timepoints")})
setGeneric("create_timepoints_contrasts", 
           function(object,...) { standardGeneric("create_timepoints_contrasts")})
setGeneric("create_diff_contrasts", 
                      function(object,...) { standardGeneric("create_diff_contrasts")})


#' Fit weekly differential expression analysis
#'
#' @inheritParams DE_timecourse
#' @param add_factors A character vector of additional variables to add to the 
#' design. See details. 
#' @return A \code{data.frame} with four columns for each of the contrasts
#'   given in \code{contrasts}, corresponding to the raw p-value of the contrast
#'   for that gene (\code{_pval}), the adjusted p-value (\code{_qval}),
#'  the t-statistic of the contrast (\code{_stat), and the
#'   estimate of log-fold-change (\code{_lfc}). The adjusted p-values are
#'   FDR-adjusted based on the Benjamini-Hochberg method, as implemented in
#'   \code{\link[stats]{p.adjust}}. The adjustment is done across all p-values
#'   for all contrasts calculated.
#' @aliases create_timepoints_contrasts DE_timepoints,Moanin-method
#' @aliases create_timepoints_contrasts,Moanin-method
#' @name DE_timepoints
#' @importFrom edgeR DGEList calcNormFactors
#' @importFrom limma voom lmFit contrasts.fit eBayes
#' @details By default the formula fitted for each gene is
#' \preformatted{   
#'    ~ Group*Timepoint +0
#' }
#' If the user gives values to \code{add_factors}, then the vector of character 
#' values given in \code{add_factors} will be \emph{added} to the default formula. 
#' So that \code{add_factors="Replicate"} will change the formula to
#' \preformatted{   
#'    ~ Group*Timepoint +0 + Replicate
#' }
#' This allows for a small amount of additional complexity to control 
#' for other variables. Users should work directly with limma for 
#' more complex models. 
#' @details If \code{use_voom_weights=TRUE}, the data is given directly to limma
#'   via \code{assay(object)}. The specific series of
#'   calls is:
#' \preformatted{   
#'    y <- edgeR::DGEList(counts=assay(object))
#'    y <- edgeR::calcNormFactors(y, method="upperquartile")
#'    v <- limma::voom(y, design, plot=FALSE)
#'    v <- limma::lmFit(v) 
#'    }
#' @details If the user set \code{log_transform=TRUE} in the creation of the
#'   \code{Moanin} object, this will not have an impact in the analysis if
#'   \code{use_voom_weights=TRUE}. Only if \code{use_voom_weights=FALSE} will
#'   this matter, in which case the log of the input data will be given to a
#'   regular call to \code{limma}:
#' \preformatted{
#'    y<-get_log_data(object)
#'    v <- limma::lmFit(y, design)
#' }
#' @examples 
#' data(exampleData)
#' moanin <- create_moanin_model(data=testData, meta=testMeta)
#' # compare groups within each timepoint
#' contrasts <- create_timepoints_contrasts(moanin,"C", "K",
#'    type="per_timepoint_group_diff")
#' head(contrasts)
#' deTimepoints=DE_timepoints(moanin, 
#'     contrasts=contrasts, use_voom_weights=FALSE)
#' head(deTimepoints)
#' # Control for replicate variable:
#' deTimepoints=DE_timepoints(moanin, 
#'     contrasts=contrasts, add_factors="Replicate",
#'     use_voom_weights=FALSE)
#' head(deTimepoints)
#'
#' # compare adjacent timepoints within each group
#' contrastsDiff <- create_timepoints_contrasts(moanin,"C",
#'    type="per_group_timepoint_diff")
#' deDiffTimepoints=DE_timepoints(moanin, 
#'     contrasts=contrastsDiff,
#'     use_voom_weights=FALSE)
#' # provide the sets of timepoints to compare:
#' contrastsDiff2<-create_timepoints_contrasts(moanin,"C",
#'    timepoints_before=c(72,120),timepoints_after=c(168,168),
#'    type="per_group_timepoint_diff")
#' deDiffTimepoints2=DE_timepoints(moanin, 
#'     contrasts=contrastsDiff2,
#'     use_voom_weights=FALSE)
#'
#' # Compare selected timepoints across groups. 
#' # This time we also return format="data.frame" which helps us keep track of
#' # the meaning of each contrast. 
#' contrastsGroupDiff<-create_timepoints_contrasts(moanin,"C", "K",
#'    timepoints_before=c(72,120),timepoints_after=c(168,168),
#'    type="group_and_timepoint_diff",format="data.frame")
#' head(contrastsGroupDiff)
#' deGroupDiffTimepoints=DE_timepoints(moanin, 
#'     contrasts=contrastsGroupDiff$contrasts,
#'     use_voom_weights=FALSE)
#' @export
setMethod("DE_timepoints","Moanin",
           function(object,
                    contrasts,add_factors=NULL,
                    use_voom_weights=TRUE){
    designText<-"~WeeklyGroup + 0"
    if(!is.null(add_factors)){
        designText<-paste(designText,"+",
            paste(add_factors,collapse="+"))
    }
    designFormula<-stats::as.formula(designText)
    design <- try(stats::model.matrix(designFormula, data=colData(object)), silent=TRUE)
    if( inherits(design, "try-error")){
        stop("Error in creating the design matrix. Error:\n",design)
    }
    
    cleaned_colnames <- gsub("WeeklyGroup", "", colnames(design))
    colnames(design) <- cleaned_colnames
    
    allcontrasts <- limma::makeContrasts(
        contrasts=contrasts,
        levels=design)
    
    if(use_voom_weights){
        y <- edgeR::DGEList(counts=assay(object))
        y <- edgeR::calcNormFactors(y, method="upperquartile")
        v <- limma::voom(y, design, plot=FALSE)
        v <- limma::lmFit(v)
    }else{
        y<-get_log_data(object)
        v <- limma::lmFit(y, design)
    }
    
    fit <- limma::contrasts.fit(v, allcontrasts)
    fit <- limma::eBayes(fit)
    contrast_names <- colnames(fit$p.value)
    fit$adj.p.value <- stats::p.adjust(fit$p.value, method="BH")
    dim(fit$adj.p.value) <- dim(fit$p.value)
    colnames(fit$adj.p.value) <- contrast_names
    combine_results <- function(ii, fit2){
        contrast_formula <- contrasts[ii]
        de_analysis <- data.frame(row.names=row.names(object))
        
        base_colname <- gsub(" ", "", contrast_formula, fixed=TRUE)
        colname_pval <- paste(base_colname, "_pval", sep="")
        colname_qval <- paste(base_colname, "_qval", sep="")
        colname_lfc <- paste(base_colname, "_lfc", sep="")
        colname_stat <- paste(base_colname, "_stat", sep="")
        
        de_analysis[colname_pval] <- fit2$p.value[, contrast_formula]
        de_analysis[colname_qval] <- fit2$adj.p.value[,  contrast_formula]
        tt <- limma::topTable(
            fit2, coef=ii, number=length(rownames(fit2$coef)),
            p.value=1, adjust.method="none", sort.by="none",
            genelist=rownames(fit2$coef))
        de_analysis[colname_stat] <- tt$t
        de_analysis[colname_lfc] <- tt$logFC
        return(de_analysis)
    }
    all_results <- do.call("cbind",
                          lapply(seq_along(contrast_names),
                                 combine_results, fit2=fit))
    return(all_results)
}
)

#' Creates pairwise contrasts for all timepoints
#'
#' @param group1 First group to consider in making contrasts, character value
#'   that must match a value of the grouping variable contained in
#'   \code{moanin_model}.
#' @param group2 Second group to consider in making contrasts, character value
#'   that must match a value of the grouping variable contained in
#'   \code{moanin_model}, unless type=="per_group_timepoint_diff", in which case
#'   should be NULL (only \code{group1} is used in comparison)
#' @param timepoints vector of timepoints to compare. Must be contained in the
#'   \code{time_variable} of the \code{moanin} object.
#' @param timepoints_after for \code{type} equal to
#'   \code{"per_group_timepoint_diff"} or, \code{"group_and_timepoint_diff"},
#'   the set of timepoints to compare, see details. By default, taken from the
#'   \code{timepoints} variable.
#' @param timepoints_before for \code{type} equal to
#'   \code{"per_group_timepoint_diff"} or, \code{"group_and_timepoint_diff"},
#'   the set of timepoints to compare, see details. By default, taken from the
#'   \code{timepoints} variable.
#' @param format the choice of "vector" (the default) for
#'   \code{create_timepoints_contrasts} returns just the character vector of
#'   contrasts. If instead \code{format="data.frame"} then a data.frame is
#'   return that identifies the timepoint and group comparisons involved in each
#'   contrast. If this is the desired output, then the input to
#'   \code{DE_timepoints} should be the column corresponding to the contrast.
#'   See examples.
#' @param type the type of contrasts that should be created. See details.
#' @details \code{create_timepoints_contrasts} creates the needed contrasts for
#'   comparing groups or timepoints in the format needed for
#'   \code{DE_timepoints} (i.e. \code{\link[limma]{makeContrasts}}), to which the
#'   contrasts are ultimately passed. The time points and groups are determined
#'   by the levels of the \code{grouping_variable} and the values of
#'   \code{time_variable} in the \code{moanin_object} provided by the user.
#' @details Three different types of contrasts are created:
#'   \itemize{
#'  \item{"per_timepoint_group_diff"}{Contrasts that compare the groups within a
#'  timepoint}
#'  \item{"per_group_timepoint_diff"}{Contrasts that compare two timepoints
#'  within a group}
#'  \item{"group_and_timepoint_diff"}{Contrasts that compare the
#'   difference between two timepoints between two levels of the
#'   \code{group_variable} of the \code{Moanin} object. These are contrasts in
#'   the form (TP i - TP (i-1))[Group1] - (TP i - TP (i-1))[Group2].}
#' }
#' @export
#' @return \code{create_timepoints_contrasts}: a character vector with each
#'   element of the vector corresponding to a contrast to be compared.
#' @seealso \code{\link[limma]{makeContrasts}}
#' @rdname DE_timepoints
#' @importFrom utils head tail
#' @export
setMethod("create_timepoints_contrasts","Moanin",
 function(object, group1, group2=NULL, 
     type=c("per_timepoint_group_diff","per_group_timepoint_diff",
            "group_and_timepoint_diff"),
     timepoints=sort(unique(time_variable(object))),
     timepoints_before=head(sort(timepoints),-1),
     timepoints_after=tail(sort(timepoints),-1),
     format=c("vector","data.frame")
     ){
    type<-match.arg(type)
    format<-match.arg(format)
    if(type=="per_timepoint_group_diff"){
        if(is.null(group2)) 
            stop("cannot choose type='per_timepoint_group_diff' and give a NULL value for argument `group2`")
        if(!all(timepoints%in% time_variable(object))) 
            stop("timepoints must consist only of timepoints in the time_variable of Moanin object")
        
        contrasts<-pertimepoint_contrast(object=object, group1=group1,
            group2=group2,timepoints=timepoints)
    }
    if(type=="group_and_timepoint_diff"){
        if(is.null(group2)) 
            stop("cannot choose type='group_and_timepoint_diff' and give a NULL value for argument `group2`")
        if(!all(timepoints_before %in% time_variable(object))) 
            stop("timepoints_before must consist only of timepoints in the time_variable of Moanin object")
        if(!all(timepoints_after %in% time_variable(object))) 
            stop("timepoints_after must consist only of timepoints in the time_variable of Moanin object")
        contrasts<-timepointdiff_contrasts(object=object, group1=group1, 
            group2=group2, timepoints_before=timepoints_before,
            timepoints_after=timepoints_after)
    }
    if(type=="per_group_timepoint_diff"){
        if(!is.null(group2)) 
            stop("cannot choose type='per_group_timepoint_diff' and give a value for argument `group2`")
        contrasts<-timepointdiff_contrasts(object=object, group1=group1, 
            group2=NULL, timepoints_before=timepoints_before,
            timepoints_after=timepoints_after)
    }
    if(format=="vector") return(contrasts$contrasts)
    else{
        return(contrasts)
    }
})
         
pertimepoint_contrast<-function(object, group1, group2, 
     timepoints){
    object <- object[,group_variable(object) %in% c(group1, group2)]
    all_timepoints <- sort(unique(time_variable(object)))
    timepoints<-timepoints[.which_timepoints(timepoints,all_timepoints,argname="timepoints")]
    contrasts <- rep(NA, length(timepoints))
    msg<-""
    foundMissing<-FALSE
    for(i in seq_along(timepoints)){
        # First, check that the two conditions have been sampled for this
        # timepoint
        timepoint <- timepoints[i]
        submeta <- object[,time_variable(object) == timepoint]
        if(length(unique(time_by_group_variable(submeta))) == 2){
            contrasts[i] <- paste0(group1, ".", timepoint, "-", group2, ".", 
                                  timepoint)
        }else if(length(unique(time_by_group_variable(submeta))) == 1){
            if(unique(group_variable(submeta))[1] ==  group1){
                missing_condition <- group2
            }else{
                missing_condition <- group1
            }
            msg <- paste0(msg,paste("timepoint",
                                   timepoint, "is missing in condition", 
                                   missing_condition,"\n"))
            foundMissing<-TRUE
        }
    }
    if(foundMissing) warning(msg)
    timepoints<-timepoints[!is.na(contrasts)]
    contrasts<-contrasts[!is.na(contrasts)]
    return(data.frame("contrasts"=contrasts,"timepoints"=as.character(timepoints),"group"=paste0(group1,"-",group2)))
 }

.which_timepoints<-function(timepoints, possibles,argname){
    wh<-which(timepoints%in% possibles)
    if(!all(timepoints%in% possibles)){
        if(length(wh)>0){
            warning("removing timepoints in ",argname," not measured for these groups\n")
        }
        else{
            stop("None of the requested timepoints measured for these groups")
        }
    }
    return(wh)
}

timepointdiff_contrasts<-function(object, group1, group2, 
    timepoints_before=NULL,timepoints_after=NULL){
    object <- object[,group_variable(object) %in% c(group1, group2)]
    all_timepoints <- sort(unique(time_variable(object)))
    
    ### Checks for timepoints
    if((is.null(timepoints_before) & !is.null(timepoints_after)) ||
    (!is.null(timepoints_before) & is.null(timepoints_after))){
        stop("either timepoints_before and timepoints_after must be given, or both must be NULL")
    }
    if(is.null(timepoints_before)){
        timepoints_before<-head(all_timepoints,-1)
        timepoints_after<-tail(all_timepoints,-1)
    }
    wh_before<-.which_timepoints(timepoints_before,all_timepoints,"timepoints_before")
    wh_after<-.which_timepoints(timepoints_after,all_timepoints,"timepoints_after")
    wh<-intersect(wh_before,wh_after)
    timepoints_before<-timepoints_before[wh]
    timepoints_after<-timepoints_after[wh]
    
    contrasts <- rep(NA, length(timepoints_before))
    # Will give a tally of timepoint pairs can't do
    msg<-""
    foundMissing<-FALSE
    for(i in seq_along(timepoints_before)){
        tpbefore <- timepoints_before[i]
        tpafter<-timepoints_after[i]
        
        # First, check that the two conditions have been sampled 
        # for both timepoints
        # Could do all at once, but not worth effort
        combos<-expand.grid(tp=c(tpbefore,tpafter),
            groups=c(group1,group2))
        npercombo<-sapply(1:nrow(combos),function(i){
            tp<-combos[i,1]
            gp<-as.character(combos[i,2])
            sum(time_variable(object)==tp & group_variable(object)==gp)
        })
        if(any(npercombo==0)){
            msg <- paste(msg,"Cannot compare",tpbefore,"and",tpafter,
                "because one of the timepoints is missing in one of the conditions.\n")
            foundMissing<-TRUE
        }else{
          if(!is.null(group2))
              contrasts[i] <- paste0(group1, ".", tpafter, "-", group1, ".", tpbefore,"-",group2,".",tpafter,"+",group2,".",tpbefore)
          else
              contrasts[i] <- paste0(group1, ".", tpafter, "-", group1, ".", tpbefore)
        } 
    }
    if(foundMissing) warning(msg)
    timepoints_before<-timepoints_before[!is.na(contrasts)]
    timepoints_after<-timepoints_after[!is.na(contrasts)]
    if(!is.null(group2)) group<-paste0(group1,"-",group2) else group<-group1
    contrasts<-contrasts[!is.na(contrasts)]
    timepoints<-paste0(timepoints_after,"-",timepoints_before)
    return(data.frame("contrasts"=contrasts,"timepoints"=as.character(timepoints),"group"=group))
    
 }


#' Creates barplot of results of per-timepoint comparison
#'
#' @param de_results results from \code{\link{DE_timepoints}}
#' @param type type of p-value to count ("qval" or "pval")
#' @param labels labels to give each bar
#' @param threshold cutoff for counting gene as DE
#' @param xlab x-axis label
#' @param ylab y-axis label
#' @param main title of plot
#' @param ... arguments passed to \code{\link{barplot}}
#' @details \code{create_timepoints_contrasts} creates the needed contrasts for
#'   comparing two groups for every timepoint in the format needed for
#'   \code{DE_timepoints} (i.e. \code{\link[limma]{makeContrasts}}, to which the
#'   contrasts are ultimately passed). The time points are determined by the
#'   meta data in the \code{moanin_object} provided by the user.
#' @return This is a plotting function, and returns (invisibly) the results of 
#'   \code{\link{barplot}}
#' @aliases perWeek_barplot
#' @examples 
#' data(exampleData)
#' moanin <- create_moanin_model(data=testData, meta=testMeta)
#' contrasts <- create_timepoints_contrasts(moanin, "C", "K")
#' deTimepoints <- DE_timepoints(moanin, 
#'     contrasts=contrasts, use_voom_weights=FALSE)
#' perWeek_barplot(deTimepoints)
#' @export
perWeek_barplot <- function(de_results, type=c("qval","pval"),
                          labels=NULL, threshold=0.05,
                          xlab="Timepoint", ylab="Number of DE genes", main="", 
                           ...){
    
    type <- match.arg(type)
    qval_colnames <- colnames(de_results)[
        grepl(type, colnames(de_results))]
    if(is.null(labels)){
        stringReplace <- paste0("_",type)
        labels <- vapply(
            strsplit(gsub(stringReplace, "", qval_colnames), "\\."),
            FUN=function(x){.subset2(x, 3)},
            character(1))
    }
    number_de_genes_per_time <- matrixStats::colSums2(
        de_results[, qval_colnames] < threshold)
    
    graphics::barplot(number_de_genes_per_time, 
            names.arg=labels, xlab=xlab,ylab=ylab,
            main=main, ...)  
}

#' Internal Validation Checks
#' @keywords internal
#' @name internal
#' @return Does not return anything. Only hits errors if there are problems.
check_data_meta <- function(data, object){
    if(!inherits(object,"Moanin")) stop("Internal coding error: passed object",
        "is not of class Moanin")
    if(is.null(dim(data))){
       data<-matrix(data,nrow=1) 
    }
    dim_data <- dim(data)
    dim_meta <- dim(object)
    if(dim_meta[2] != dim_data[2]){
        stop(
            "User-given data and Moanin object are inconsistent. Data is has ", 
            ncol(data),"columns; Moanin object has", ncol(object))
    }

}


#' Check is 2D
#'
#' @keywords internal
#' @rdname internal
check_is_2d <- function(X){
    dim_data <- dim(X)
    if(is.null(dim_data)){
        stop("Data is expected to be 2D. No dimension found.")
    }
}


#' Check input are contrasts
#'
#' Will check that the contrasts provided are indeed contrasts. contrasts are
#'  expected to be either a vector of string or a matrix containing the
#'  contrasts coefficients.
#'  
#' @details If a vector of string is provided, the function will call
#'  limma::makeContrast in order to obtain the contrasts coefficients.
#'
#' @details If a contrasts matrix is provided, it will perform a number of
#'   checks on the contrasts matrix to make sure it contains the number of rows
#'   expected, and that each contrast indeed sums to 0.
#' @returns \code{is_contrasts} returns the contrasts, with any corrections.
#' @keywords internal
#' @rdname internal
is_contrasts <- function(contrasts, moanin_model){
    if(!inherits(moanin_model, "Moanin") ) 
        stop("Coding error: internal function is_contrasts expect class",
            "Moanin object")
    if(is.vector(contrasts)){
        # XXX Should we add more tests here in order to provide meaningful
        # error messages?
        contrasts <- limma::makeContrasts(
            contrasts=contrasts,
            levels=levels(group_variable(moanin_model)))
    }else{
        # Basic checks to make sure we have contrasts
        dim_contrasts <- dim(contrasts)
        if(is.null(dim_contrasts)){
            msg <- "Contrasts need to be either a vector of string or a 2D matrix"
            stop(msg)
        }
        
        n_groups <- length(levels(group_variable(moanin_model)))
        if(dim(contrasts)[1] != n_groups){
            msg <- paste(
                "When contrasts provided is a matrix, it should contain the",
                "same number of rows as the number of groups. Provided:",
                dim(contrasts[1]), "expected:", n_groups)
            stop(msg)
        }
        
        if(any(matrixStats::colSums2(contrasts) != 0)){
            msg <- paste("When contrasts provided is",
                "matrix, all columns should sum to 0.")
            stop(msg)
        }
    }

    return(contrasts) 
}

data(exampleData)
context("moanin::validation")


test_that("validation:check_data_meta", {
    moanin_model = create_moanin_model(data=testData,meta=testMeta)
    
    # Running check_data_meta should work fine
    expect_silent(check_data_meta(testData, moanin_model))

    expect_error(
	check_data_meta(testData[, 2:10], moanin_model))
})


test_that("validation:check_is_2d", {
    data_2d = matrix(1:9, nrow=3)
    expect_silent(check_is_2d(data_2d))
    expect_silent(check_is_2d(as.data.frame(data_2d)))
    expect_error(check_is_2d(1:9))
})


test_that("validation:is_contrasts", {
    contrasts_formula = c("M-K", "M-C")
    moanin_model = create_moanin_model(data=testData,meta=testMeta)
    expect_silent(contrasts <-
	    is_contrasts(contrasts_formula, moanin_model))
    expect_silent(is_contrasts(contrasts,  moanin_model))

    contrasts = limma::makeContrasts(
	    contrasts=contrasts_formula, levels=levels(testMeta$Group))
    expect_silent(
	    is_contrasts(contrasts,  moanin_model))

})

#'Compute consensus matrix from labels
#'
#'@param labels a matrix with each column corresponding to a the results of a
#'  single clustering routine. Each column should give the cluster assignment
#'  FIXME: What is the required format of entries??
#'@param scale  boolean, optional, default: TRUE. Whether to rescale the
#'  resulting consensus matrix so that entries correspond to proportions.
#'@return a symmetric matrix of size NxN, where N is the number of rows of the
#'  input matrix \code{labels}. Each i,j entry of the matrix corresponds the
#'  number of times the two rows were in the same cluster across the clusterings
#'  (\code{scale=FALSE}) or the proportion of clustering that the two rows are
#'  in the same cluster (\code{scale=TRUE}).
#' @examples 
#' data(exampleData)
#' moanin <- create_moanin_model(data=testData,meta=testMeta)
#' #small function to run splines_kmeans on subsample of 50 genes
#' subsampleCluster<-function(){
#'    ind<-sample(1:nrow(moanin),size=50,replace=FALSE)
#'    km<-splines_kmeans(moanin[ind,],n_clusters=3)
#'    assign<-splines_kmeans_predict(moanin,km, 
#'       method="distance")
#'   }
#' kmClusters=replicate(10,subsampleCluster())
#' cm<-consensus_matrix(kmClusters)
#' heatmap(cm)
#' @export
consensus_matrix <- function(labels, scale=TRUE){
    melted_labels <- reshape2::melt(as.matrix(labels))
    colnames(melted_labels) <- c("Gene", "Clustering", "Label")
    melted_labels$Clustering_lab <- 
        as.factor(melted_labels$Clustering):as.factor(melted_labels$Label)
    w <- reshape2::dcast(melted_labels, Gene~Clustering_lab, 
                        value.var="Clustering_lab", fun.aggregate=length)
    x <- as.matrix(w[,-1])
    x[is.na(x)] <- 0
    x <- apply(x, 2,  function(x) as.numeric(x > 0))
    consensus <- tcrossprod(x) 
    if(scale){
        # EAP: Slightly faster:
        diag_elements <- matrix(diag(consensus),
                        nrow=nrow(consensus),ncol=ncol(consensus))
        consensus <- consensus/diag_elements
        consensus <- consensus/t(diag_elements)
        diag(consensus) <- 0
    }
    return(consensus)
}

#' @details For each element of the list \code{labels},
#'   \code{plot_cdf_consensus} calculates the consensus between the clusterings
#'   in the matrix, i.e. the number of times that pairs of rows are in the same
#'   cluster for different clusterings (columns) of the matrix using the
#'   \code{\link{consensus_matrix}} function. Then the set of values (the N(N-1)
#'   values in the upper triangle of the matrix), are converted into a cdf
#'   function and plotted.
#' @return \code{plot_cdf_consensus} invisibily returns list of the upper
#'   triangle values, with the list of same length as that of \code{labels}.
#' @rdname get_auc_similarity_scores
#' @export
plot_cdf_consensus <- function(labels){
    all_labels <- labels
    if(is.null(names(all_labels))) 
        names(all_labels)<-paste("Set",seq_along(all_labels))
    n_clusters <- names(all_labels)
    
    colors <- grDevices::rainbow(length(n_clusters))
    xrange <- c(0, 1)
    yrange <- c(0, 1)
    
    graphics::plot(xrange, yrange, type="n")
    
    cdfList<-list()
    for(i in seq_along(n_clusters)){
        cluster <- n_clusters[i]
        color <- colors[i]
        
        labels <- all_labels[[cluster]]
        consensus <- consensus_matrix(labels, scale=FALSE)
        consensus <- consensus / max(consensus)
        consensus <- sort(consensus[upper.tri(consensus)])
        x_axis <- seq_along(consensus) / length(consensus)
        cdfList<-c(cdfList,list(consensus))
        graphics::lines(consensus, x_axis, type="b",
            pch=16,
            col=color,
            lwd=1)
    }
    
    graphics::legend("bottomright", legend=n_clusters,
            lty=rep(1, length(n_clusters)),
            pch=rep(16, length(n_clusters)),
            col=colors,
            title="Clusters", text.font=4)
    invisible(cdfList)
}

#' Evaluate the consensus between sets of clusterings
#'
#' @description Methods for evaluating the consensus between sets of
#'   clusterings, usually in the context of subsetting of the data or different
#'   numbers of clusters.
#' @param labels a list. Each element of
#'   the list is a matrix that gives the results of a clustering routine in each
#'   column (see \code{\link{consensus_matrix}}). Usually each column would be
#'   the result of running the clustering on a subsample or bootstrap resample
#'   of the data.
#' @param method method for calculation of similarity for the AUC measure, one
#'   of "consensus" or "nmi". See details.
#' @details For each set of clusterings given by \code{labels} (i.e. for each
#'   matrix \code{M} which is an element of the list \code{labels})
#'   \code{get_auc_similarity_scores} calculates a pairwise measure of
#'   similarity between the columns of \code{M}. These pairwise scores are
#'   plotted against their rank, and the final AUC measure is the area under
#'   this curve.
#' @details For method "consensus", the pairwise measure is given by calculating
#'   the consensus matrix using \code{\link{consensus_matrix}} with
#'   \code{scale=FALSE}. The consensus matrix is divided by the max of \code{M}.
#' @details For method "nmi", the pairwise value is the NMI value between each
#'   pair of columns of the matrix of clusterings using the
#'   \code{\link[NMI]{NMI}} function.
#' @seealso \code{\link{consensus_matrix}}, \code{\link[NMI]{NMI}},
#'   \code{\link{plot_cdf_consensus}}
#' @returns \code{get_auc_similarity_scores} returns a vector, equal to length
#'   of the list \code{labels}, giving the AUC value for each element of
#'   \code{labels}.
#' @aliases plot_cdf_consensus plot_model_explorer
#' @examples 
#' data(exampleData)
#' moanin <- create_moanin_model(data=testData,meta=testMeta)
#' #small function to run splines_kmeans on subsample of 50 genes
#' subsampleCluster<-function(){
#'    ind<-sample(1:nrow(moanin),size=50)
#'    km<-splines_kmeans(moanin[ind,],n_clusters=3)
#'    assign<-splines_kmeans_score_and_label(moanin, km, 
#'        proportion_genes_to_label=1.0)$label
#' }
#' kmClusters1=replicate(10,subsampleCluster())
#' kmClusters2=replicate(10,subsampleCluster())
#' # Note, because of the small number of replicates (10), 
#' # these plots are not representative of what to expect.
#' out<-plot_cdf_consensus(labels=list(kmClusters1,kmClusters2)) 
#' get_auc_similarity_scores(list(kmClusters1,kmClusters2))
#' plot_model_explorer(list(kmClusters1,kmClusters2))
#' @export
get_auc_similarity_scores <- function(labels, method=c("consensus", "nmi")){
    method<-match.arg(method)
    
    all_labels <- labels
    if(is.null(names(all_labels))) 
        names(all_labels)<-paste("Set",seq_along(all_labels))
    n_clusters <- names(all_labels)
    
    auc_scores <- rep(0, length(n_clusters))
    for(i in seq_along(n_clusters)){
        cluster <- n_clusters[i]
        labels <- all_labels[[cluster]]
        if(method == "consensus"){
            consensus <- consensus_matrix(labels, scale=FALSE)
            consensus <- consensus / max(consensus)
            scores <- consensus[upper.tri(consensus)]
        }else if(method == "nmi"){
            scores <- get_nmi_scores(labels) 
        }
        scores <- sort(scores)
        y_axis <- seq_along(scores) / length(scores)
        auc_score <- sum(diff(scores) * zoo::rollmean(y_axis, 2))
        
        auc_scores[i] <- auc_score
    }
    return(auc_scores)
}


get_nmi_scores <- function(labels){
    
    nmi <- function(x, y){
        x_dataframe <- as.data.frame(x)
        colnames(x_dataframe) <- c("Label")
        x_dataframe$Gene <- row.names(x_dataframe)
        x_dataframe <- x_dataframe[c("Gene", "Label")]
        
        y_dataframe <- as.data.frame(y)
        colnames(y_dataframe) <- c("Label")
        y_dataframe$Gene <- row.names(y_dataframe)
        y_dataframe <- y_dataframe[c("Gene", "Label")]
        
        return(NMI::NMI(x_dataframe, y_dataframe))
    }
    
    n_trials <- dim(labels)[2]
    scores <- NULL
    for(trial in seq_len(n_trials)){
        if(trial == n_trials){
            break
        }
        column <- colnames(labels)[trial]
        columns_to_consider <- (trial+1):n_trials
        label <- labels[,trial]
        scores <- c(scores, as.vector(
            unlist(lapply(labels[, columns_to_consider, drop=FALSE], 
                          function(x){nmi(x, label)}))))
    }
    return(scores)
}


#' Plot model explorer
#'
#' @param colors a vector of colors, of length equal to the length of
#'   \code{labels}
#' @return This function is a plotting function does not return anything
#' @rdname get_auc_similarity_scores
#' @export
#' @importFrom grDevices rainbow
plot_model_explorer <- function(labels, colors=rainbow(length(labels))){
    all_labels <- labels
    if(is.null(names(all_labels))){ 
        names(all_labels) <- paste("Set",seq_along(all_labels))
    }
    n_clusters <- names(all_labels)
    if(length(colors) != length(labels)) {
        stop("colors argument should be same length as labels")
    }
    nmi_scores <- list()
    
    max_trial <- 0
    min_score <- 1
    max_score <- 0
    for(i in seq_along(n_clusters)){
        n_cluster <- n_clusters[i]
        color <- colors[i]
        
        labels <- all_labels[[n_cluster]]
        scores <- get_nmi_scores(labels)
        
        nmi_scores[[n_cluster]] <- sort(scores)
        max_trial <- max(max_trial, length(scores))
        min_score <- min(min_score, min(scores))
        max_score <- max(max_score, max(scores))
    }
    xrange <- c(min_score, max_score)
    yrange <- c(1, max_trial)
    
    graphics::plot(xrange, yrange, type="n", xlab="NMI", ylab="")
    
    for(i in seq_along(n_clusters)){
        color <- colors[i]
        scores <- nmi_scores[[i]]
        graphics::lines(sort(scores), seq_along(scores), type="b",
                        pch=16,
                        col=color,
                        lwd=1)
    }
    
    graphics::legend(min_score, length(scores)*0.9, legend=n_clusters,
                        lty=rep(1, length(scores)),
                        pch=rep(16, length(scores)),
                        col=colors, 
                        title="Clusterings", text.font=4)
}

context("moanin::cluster_validation.R")

test_that("cluster_validation::consensus_matrix", {
    set.seed(42)
    n_genes = 40
    n_clusters = 5
    n_labels = 20
    labels = sample.int(n_labels, n_clusters * n_genes, replace=TRUE)
    dim(labels) = c(n_genes, n_clusters)
    colnames(labels) = lapply(1:n_clusters, function(x){paste0("C", x)})
    row.names(labels) = lapply(1:n_genes, function(x){paste0("G", x)})

    expect_silent(consensus_matrix(labels))
})

library(moanin)
data(exampleData)
library(limma)

context("moanin::DE_timepoints.R")

test_that("DE_timepoints::create_timepoints_contrasts", {
    #change group variable to check that works everywhere
    colnames(testMeta)<-c("Index","condition","replicate","time")
    
    moanin_model = create_moanin_model(data=testData,meta=testMeta,
        group_variable_name="condition",
        time_variable_name="time")
    expect_silent(contrasts<-create_timepoints_contrasts( moanin_model,"M", "C"))
    expect_silent(contrasts<-create_timepoints_contrasts( moanin_model,"M", 
                  type="per_group_timepoint_diff"))
    expect_silent(contrasts<-create_timepoints_contrasts( moanin_model,"M", "C",
                  type="group_and_timepoint_diff"))

    # can't give two groups for this contrast
    expect_error(contrasts<-create_timepoints_contrasts( moanin_model,"M", "C",
                    type="per_group_timepoint_diff"))

    
    # Now drop timepoint 6 in condition C. That should raise a warning
    mask = !((testMeta$condition == "C") & (testMeta$time == 6))
    msg = paste0("timepoint 6 is missing in condition C")
    expect_warning(create_timepoints_contrasts(moanin_model[,mask],
        "M", "C"), msg)
        
    expect_silent( DE_timepoints(moanin_model,
         contrasts=contrasts, use_voom_weights=FALSE))
    # Create a second moanin model with counts
    moanin_model_counts = create_moanin_model(
        data=round(exp(testData)), meta=testMeta,
        log_transform=TRUE,
        group_variable_name="condition",
        time_variable_name="time")
    expect_silent( DE_timepoints(moanin_model_counts,
          contrasts=contrasts, use_voom_weights=TRUE))
    
    #Check add replicate
    expect_silent( DE_timepoints(moanin_model_counts,
          contrasts=contrasts, use_voom_weights=TRUE,
          add_factors="replicate"))

    expect_error( DE_timepoints(moanin_model_counts,
        contrasts=contrasts, use_voom_weights=TRUE,
        add_factors="Reps"),"Error in creating the design matrix")

})

library(moanin)
data(exampleData)
library(testthat)

context("moanin::visualization.R")

test_that("visualization::plot_splines_data", {
    moanin_model = create_moanin_model(data=testData,meta=testMeta)
    
    singleData = unlist(testData[1,])
    # Plot only one gene
    expect_silent(plot_splines_data(data=singleData , object=moanin_model))
    expect_silent(plot_splines_data(data=singleData , object=moanin_model, smooth=TRUE))
    expect_error(plot_splines_data(data=singleData , moanin_model,
				   smooth=TRUE, mfrow=c(1, 1),legend=TRUE))
    expect_silent(plot_splines_data(data=singleData ,  moanin_model,
				    smooth=TRUE, mfrow=c(1, 1),
				    legend=FALSE))

    # Plot several genes, using subset_data
    subset_data = row.names(testData)[2:5]
    expect_silent(plot_splines_data(data=testData, object=moanin_model,
				    subset_data=subset_data))

    # Don't provide data
    expect_silent(plot_splines_data(object=moanin_model,
                                    subset_data=subset_data))
    
    expect_error(plot_splines_data(data=testData, object=moanin_model,
				   subset_data=c(subset_data, "not_a_gene")))


    # Check that some options make sense
    expect_error(plot_splines_data(data=testData, moanin_model,
				   subset_data=subset_data,
				   mfrow=c(1, 1)))
		
    expect_error(plot_splines_data(data, moanin_model,
                                   subset_data=subset_data,
				   mfrow=c(1)))

})

library("moanin")

context("moanin::de_analysis_utils.R")
data(exampleData)

test_that("Estimating log fold change smoke tests", {
    # This is just a smoke test.
    moanin_model = moanin::create_moanin_model(data=testData,meta=testMeta)

    # Reduce the data set
    methods = eval(formals(estimate_log_fold_change)$method)

    contrast_formula = c("C-K")
    contrasts = limma::makeContrasts(contrasts=contrast_formula,
				     levels=droplevels(testMeta$Group))
    for(method in methods){
        expect_silent(
	        estimate_log_fold_change(moanin_model, contrasts, method=method))
	    expect_silent(
	        estimate_log_fold_change(moanin_model, contrast_formula, method=method))
    }

    # Now same test, but several contrasts
    contrasts = limma::makeContrasts(contrasts=c("C-K", "C-M"),
				     levels=droplevels(testMeta$Group))
    for(method in methods){
        expect_silent(
	    estimate_log_fold_change(
		    object=moanin_model, contrasts, method=method))
    }

})

test_that("Estimating log fold change with unknown error", {

    moanin_model = moanin::create_moanin_model(data=testData,meta=testMeta)
    
    # Reduce the data set
    contrasts = c("C-K")
    expect_error(estimate_log_fold_change(moanin_model,
					  contrast, method="hahaha"))
})


test_that("Estimating log fold change", {
    data<-testData
    data[, testMeta$Group == "C"] = 0
    data[, testMeta$Group == "K"] = 1
    moanin_model = moanin::create_moanin_model(data=data,meta=testMeta)
    # Reduce the data set
    contrasts = limma::makeContrasts(contrasts=c("C-K"), levels=testMeta$Group)
    lfc_max = estimate_log_fold_change(moanin_model, contrasts=contrasts, method="max")
    expect_equal(1, max(lfc_max))
    expect_equal(1, min(lfc_max))

})

data(exampleData)


context("moanin::cluster.R")

test_that("cluster::splines_kmeans", {
    random_seed = 42
    moanin_model = create_moanin_model(data=testData,meta=testMeta)
    expect_silent(moanin::splines_kmeans(moanin_model, n_init=2,
				         random_seed=random_seed))
})


test_that("cluster::splines_kmeans_score_and_label", {
    random_seed = 42
    moanin_model = create_moanin_model(data=testData, meta=testMeta)
    clustering_results = moanin::splines_kmeans(
	    moanin_model, n_init=1,
	    random_seed=random_seed, rescale=TRUE) 

    expect_silent(scores_and_labels<-splines_kmeans_score_and_label(moanin_model, 
                               clustering_results))
    expect_equal(row.names(testData), row.names(scores_and_labels$scores))

    expect_silent(
        scores_and_labels2 <- splines_kmeans_score_and_label(
            moanin_model,
            data=testData, clustering_results))
    expect_equal(scores_and_labels2, scores_and_labels)
    
    # Set a max score that we know is belove the max score found automatically
    max_score = max(scores_and_labels$scores, na.rm=TRUE) / 2
    scores_and_labels = splines_kmeans_score_and_label(
	    object=moanin_model,
        data=testData, clustering_results, max_score=(max_score))
    labels = scores_and_labels$labels
    scores = apply(
        scores_and_labels$scores[!is.na(labels), ],
        1,
        function(x){min(x, na.rm=TRUE)})
    expect_true(max(scores) <= max_score)

    # Now, just do a ghost test with the rescale_separately_on

    expect_silent(
        splines_kmeans_score_and_label(
            moanin_model,
            clustering_results, rescale_separately=TRUE))

})

library(moanin)
library(limma)
data(exampleData)
context("moanin::de_analysis.R")

test_that("time-course DE analysis", {
    #change group variable to check that works everywhere
    colnames(testMeta)<-c("Index","condition","replicate","time")
    
    moanin_model = create_moanin_model(data=testData, 
        meta=testMeta,
        group_variable_name="condition",
        time_variable_name="time")
    # Create a second moanin model with counts
    moanin_model_counts = create_moanin_model(
        data=round(exp(testData)), meta=testMeta,
        log_transform=TRUE,
        group_variable_name="condition",
        time_variable_name="time")

    # Do only one contrast
    contrast_formulas = c("K-M")
    contrast = limma::makeContrasts(
	    contrasts=contrast_formulas,
	    levels=levels(droplevels(testMeta$condition)))

    expect_silent(moanin::DE_timecourse(
        moanin_model, contrasts=contrast,
	    use_voom_weights=FALSE))

    expect_silent(moanin::DE_timecourse(
        moanin_model_counts, contrasts=contrast,
	    use_voom_weights=TRUE))

    # Now test with several contrasts
    contrast_formulas = c("K-M", "C-M")
    contrasts = limma::makeContrasts(
	    contrasts=contrast_formulas,
	    levels=levels(droplevels(testMeta$condition)))

    expect_silent(moanin::DE_timecourse(moanin_model, contrasts=contrast,
	    use_voom_weights=FALSE))
        
    expect_silent(moanin::DE_timecourse(
        moanin_model_counts, contrasts=contrast,
	    use_voom_weights=TRUE))

    
    # Check that the names of the columns make sense.
    de_results = moanin::DE_timecourse(
	    moanin_model, contrasts=contrast,
	    use_voom_weights=FALSE)

    })

---
title: "moanin: An R Package for Time Course RNASeq Data Analysis"
output: BiocStyle::html_document
vignette: >
  %\VignetteEngine{knitr::knitr}
    %\VignetteIndexEntry{The Moanin Package}
      %\usepackage[UTF-8]{inputenc}
---
```{r echo=FALSE, include=FALSE}
knitr::opts_chunk$set(
  cache=TRUE, autodep=TRUE, warning=FALSE, error=FALSE, message=FALSE,
  echo=TRUE, out.width=".5\\textwidth",
  duplicate.label="allow", fig.pos <- "H", out.extra = "")
```

The package moanin was developed to provide a simple and efficient workflow
for long time-course gene expression data. It makes it simple to do
differential expression between conditions based on either individual
condition comparisons per time point, or by fitting spline functions per gene.
There are also functions to help with clustering and visualization.

We do not detail all of the features of the package here, but some of the main
functions. See our more in-depth workflow paper at
https://github.com/NelleV/2019timecourse-rnaseq-pipeline. 

# Setup

We will work with time course data available in the accompanying
`timecoursedata` package. This is mRNA-Seq timecourse data on a plant,
sorghum. The plants were sampled every week for 17 weeks. The plants were of
two different varieties (i.e. different genotypes), and under three different
watering conditions:

* Control: regular weekly watering
* Preflowering drought: plants under drought under pre-flowering (weeks before
  week 9), and watered like control for week 9 and afterward.
* Postflowering drought: plants were regularly watered pre-flowering (weeks
  through week 9), and then watering withheld weeks 10 onward.

This data is available on both the leaf and the root sampled from the same
plant. We will concentrate on the leaf samples, available as a dataset
`varoquaux2019leaf`.  

```{r}
library(moanin)
library(timecoursedata)
data(varoquaux2019leaf)
names(varoquaux2019leaf)
```

The `data` element contains the gene expression data, while the `meta` data
consists of information regarding the samples.


For simplicity, we are going to focus on comparing the three different
watering conditions within the "BT642" variety (the variety known for
comparative tolerance of pre-flowering drought). We are also going to drop
Week 2, which is only measured in the control samples.

```{r}
whSamples<-with(varoquaux2019leaf$meta,which(Genotype=="BT642" & Week >2))
preData<-varoquaux2019leaf$data[,whSamples]
preMeta<-varoquaux2019leaf$meta[whSamples,]
dim(preData)
dim(preMeta)
```

# Set up `moanin` class

The first step is to create a moanin class, which uses our meta data to create
the needed information for future analysis. Specifically, in addition to
storing the meta information, the function `create_moanin_model` will define a
splines basis for future calculations. By default the model is given as a
separate spline basis for each level of a factor `Group`, plus a group
intercept with the following R formula 

`~Group:ns(Timepoint, df=4) + Group + 0`


```{r}
moaninObject <- create_moanin_model(data=preData, meta=preMeta, 
                            group_variable="Condition", time_variable="Week")
moaninObject
```

## Moanin and SummarizedExperiment

`Moanin` extends the `SummarizedExperiment` class, and as such the data as
well as the meta data are saved in one object. This means you can index the
`Moanin` object like a regular matrix, and safely be indexing the related meta
data:

```{r}
moaninObject[,1:2]
```

## Log-tranformation

Because we have count data from mRNA-Seq, we set the argument to
`log_transform=TRUE`. This means that many of the functions will internally
use the transformation `log(x+1)` on the `assay(moaninObject)` for
computations and visualizations. However, for some steps of the DE process the
counts will be used (see below). The user can also just transform the data
herself,

```{r}
logMoaninObject<-moaninObject
assay(logMoaninObject)<-log(assay(moaninObject)+1)
```


# Weekly differential expression analysis

One type of DE analysis we can do is to compare our watering conditions to
each other, for every time point. We do this via a call to `limma` for the DE
analysis, but before we can do this, we need to set up the appropriate
contrasts. For example, "Preflowering - Control" (see `?makeContrasts` in the
limma package). We provide a function to do this for every week, so as to
avoid this step.

```{r}
preContrasts <- create_timepoints_contrasts(moaninObject,"Preflowering", "Control")
```

Notice we also get a warning that timepoint 16 is missing in our Control
Samples (in fact, some of these time points only have a single observations
per time point, which is not particularly appropriate for DE analysis per
week). 

We can also create contrasts to compare Postflowering and Control, and
Preflowering and Postflowering. We get many warnings here, because
Post-flowering only has samples after week 9. 

```{r}
postContrasts <- create_timepoints_contrasts(
    moaninObject, "Postflowering", "Control" )
prepostContrasts <- create_timepoints_contrasts(
    moaninObject, "Postflowering", "Preflowering")
```

We can run the DE analysis, setting `use_voom_weights=TRUE` to make use of
`limma` correction for low-counts. We will subset to just the first 500 genes
for illustration purposes, though the full set of genes does not take very
long

```{r}
weeklyPre <- DE_timepoints(moaninObject[1:500,], 
    contrasts=c(preContrasts,postContrasts,prepostContrasts),
     use_voom_weights=TRUE)
```

By setting `use_voom_weights=TRUE`, the DE step is done after an
`upperquartile` normalization via `limma` using voom weights (see
`?DE_timepoints` for details). Therefore this should only be done if the input
data to the `Moanin` class are counts (or on a count scale, such as expected
counts from TopHat); in this case the user should set `log_transform=TRUE` in
the construction of the `Moanin` object, so that other functions will take the
log appropriately.    

The results give the raw p-value (`_pval`), the FDR adjusted p-values
(`_qval`), and the estimate of log-fold-change (`_lfc`) for each week (3
columns for each of the `r ncol(weeklyPre)/3` contrasts):

```{r}
dim(weeklyPre)
head(weeklyPre[,1:10])
```

## Compare change across two time points

Different types of contrasts are available within `create_timepoint_contrasts` based on the `type` argument. Previously we used the default (`per_timepoint_group_diff`) which gives the group differences per timepoint. 

We can also use the same kind of approach to compare within a group the difference between two timepoints. Here we look at the difference between adjacent timepoints in "Postflowering", 

```{r}
preDiffContrasts <- create_timepoints_contrasts(
    moaninObject, "Preflowering" ,type="per_group_timepoint_diff")
head(preDiffContrasts)
```

We can also compare these time differences between the two groups, a contrast that takes the form of 
$$(TP i - TP (i-1))[Group1] - (TP i - TP (i-1))[Group2]$$
These contrasts we can create by setting `type="group_and_timepoint_diff"`

```{r}
preGroupDiffContrasts <- create_timepoints_contrasts(
    moaninObject, "Preflowering", "Control" ,type="group_and_timepoint_diff")
head(preGroupDiffContrasts)
```

By default the consecutive times are compared, but we could instead compare non-consecutive time points by giving an explicit vector of the timepoints to compare:

```{r}
preGroupDiffContrasts <- create_timepoints_contrasts(
    moaninObject, "Preflowering", "Control" ,
    type="group_and_timepoint_diff",timepoints_before=c(6,8),timepoints_after=c(8,9))
head(preDiffContrasts)
```

We can again submit these contrasts to `DE_timepoints`

```{r}
weeklyGroupDiffPre <- DE_timepoints(moaninObject[1:500,], 
    contrasts=preGroupDiffContrasts,
     use_voom_weights=TRUE)
head(weeklyGroupDiffPre)
```


# Time-course differential expression analysis between two groups

The results of such a weekly analysis can be quite difficult to interpret, the
number of replicates per week can be quite low, and different numbers of
replicates in different weeks can make it difficult to compare the analysis
across weeks.

An alternative approach is to fit a smooth spline to each gene per group, and
perform differential tests as to whether there are differences between the
spline functions between two groups. We provide this function in
`DE_timecourse`, where the user provides a string of comparisons that they
wish to make:

```{r}
timecourseContrasts <- c("Preflowering-Control",
			 "Postflowering-Control",
			 "Postflowering-Preflowering")
splinePre <- DE_timecourse(moaninObject[1:500,], contrasts=timecourseContrasts,
     use_voom_weights=TRUE)
```

Again, because our input data are counts, we set `use_voom_weights=TRUE` to
weight our observations based on their variability. Because we set
`log_transform=TRUE` in our creation of `moaninObject`, the function will
internally take the log of the data for fitting the splines, but use the
counts for creating the voom weights (see `?DE_timecourse`).

Each contrast has a `pval` and a `qval` column:

```{r}
head(splinePre)
```

There is not a log-fold change column, because it is more complicated to
define the log-fold change over a series of timepoints, where potentially the
means may even switch from being over-expressed to under-expressed. We provide
a function `estimate_log_fold_change` which gives the option of estimating
several kinds of log-fold-change. We demonstrate with the method `abs_sum`,
which gives the sum of the absolute difference in the means across time
points:

```{r}
log_fold_change_timepoints <- estimate_log_fold_change(
    moaninObject[1:500,], contrasts=timecourseContrasts, method="sum")
head(log_fold_change_timepoints)
```

See `?estimate_log_fold_change` to see the full set of methods.

# Visualizing Genes of Interest

The package `moanin` also provides a simple utility function
(`plot_splines_data`) to visualize gene time-course data from different
conditions. We use this to plot the 10 genes with the largest log-fold-change
in `Preflowering-Control` contrast (since we only looked at the first top 500,
these aren't representative of the overall signal in the data set, which is
much stronger).

```{r}
whSig <- which(splinePre[,"Preflowering-Control_qval"]<0.05)
deGenes <- order(abs(
    log_fold_change_timepoints)[whSig,"Preflowering-Control"],
    decreasing=TRUE)[1:10]

plot_splines_data(moaninObject[whSig, ][deGenes,],
    smooth=TRUE, mar=c(1.5,2.5,2,0.1))
```


# Clustering of time-course data

We can also cluster the data based on their spline fits. Here we would like to
work with the log transform of the counts which are stored in `moaninObject`,
so we set `log_transform=TRUE`. 

```{r clustering}
# First fit the kmeans clusters
kmeans_clusters <- splines_kmeans(moaninObject[1:500,], n_clusters=3,
    random_seed=42, 
    n_init=20)
```


We then use the `plot_splines_data`  function, only now applied to the
centroids of the kmeans clustering, to visualize the centroids of each cluster
obtained with the splines k-means model. 

```{r}
plot_splines_data(
    data=kmeans_clusters$centroids, moaninObject,
    smooth=TRUE)
```

## Assigning genes to clusters

Because there is variability in how well the genes fit a cluster, we would
like to be able to score how well a gene fits a cluster. Furthermore, we often
chose a subset of genes based on a filtering process, and we would like to
have a mechanism to assign all genes to a cluster. 

The function `splines_kmeans_score_and_label` gives a goodness-of-fit score
between each gene and each cluster. 

```{r}
scores_and_labels <- splines_kmeans_score_and_label(
    object=moaninObject,data=preData[1:2000,], kmeans_clusters=kmeans_clusters)
```

Notice that we used more genes here (first 2000), even though previously we
only used first 500 to make the clusters. These choices were unrealistic,
since in practice we would probably pick high variable genes or differentially
expressed genes, rather than the first 500, but at least give a sense of how
this works.

The result is a list with elements `scores` and `labels`. `scores` gives the
goodness-of-fit score between each gene and each cluster

```{r}
head(scores_and_labels$scores)
```

`labels` gives the best cluster assignment for each gene, but only if its
score in its best cluster is above a certain threshold (see
`?splines_kmeans_score_and_label`)

```{r}
head(scores_and_labels$labels)
# How many are not assigned a label?
sum(is.na(scores_and_labels$labels))
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_splines_data}
\alias{plot_splines_data}
\alias{plot_splines_data,Moanin,matrix-method}
\alias{plot_splines_data,Moanin,numeric-method}
\alias{plot_splines_data,Moanin,data.frame-method}
\alias{plot_splines_data,Moanin,DataFrame-method}
\alias{plot_splines_data,Moanin,missing-method}
\title{Plotting splines}
\usage{
\S4method{plot_splines_data}{Moanin,matrix}(
  object,
  data,
  colors = NULL,
  smooth = FALSE,
  legend = TRUE,
  legendArgs = NULL,
  subset_conditions = NULL,
  subset_data = NULL,
  simpleY = TRUE,
  centroid = NULL,
  scale_centroid = c("toData", "toCentroid", "none"),
  mar = c(2.5, 2.5, 3, 1),
  mfrow = NULL,
  addToPlot = NULL,
  ylab = "",
  xaxis = TRUE,
  yaxis = TRUE,
  xlab = "Time",
  ...
)

\S4method{plot_splines_data}{Moanin,numeric}(object, data, ...)

\S4method{plot_splines_data}{Moanin,data.frame}(object, data, ...)

\S4method{plot_splines_data}{Moanin,DataFrame}(object, data, ...)

\S4method{plot_splines_data}{Moanin,missing}(object, data, ...)
}
\arguments{
\item{object}{An object of class \code{\link{Moanin}}, an object containing
all related information for time course data and the splines model that
will be used (if applicable). See \code{\link{create_moanin_model}} for
more details.}

\item{data}{matrix containing the data to be plotted, where each row of the
data provided will be plotted as a separate plot. If missing, will rely on
data in \code{assay(object)}}

\item{colors}{vector, optional, default NULL. Vector of colors}

\item{smooth}{boolean, optional, default: FALSE. Whether to smooth the
centroids or not.}

\item{legend}{boolean whether to include a legend (default:TRUE)}

\item{legendArgs}{list of arguments to be passed to legend command (if
\code{legend=TRUE})}

\item{subset_conditions}{list if provided, only plots the subset of conditions
provided. Else, plots all conditions}

\item{subset_data}{list if provided, only plots the subset of data (ie, the
rows) provided. Can be any valid vector for subsetting a matrix. See
details.}

\item{simpleY}{boolean, if true, will plot all genes on same y-axis and
minimize the annotation of the y axis to only label the axis in the exterior
plots (the x-axis is always assumed to be the same across all plots and will
always be simplified)}

\item{centroid}{numeric vector (or matrix of 1 row) with data to use to fit
the splines. If \code{NULL}, the splines plotted will be from the data.}

\item{scale_centroid}{determines whether the centroid data given in
\code{centroid} should be rescaled to match that of the data
(\code{"toData"}), or the data scaled to match that of centroid
(\code{"toCentroid"}), or simply plotted as is (\code{"none"}).}

\item{mar}{vector of margins to set the space around each plot (see
\code{\link{par}})}

\item{mfrow}{a vector of integers of length 2 defining the grid of plots to be
created (see \code{\link{par}}). If missing, the function will set a value.}

\item{addToPlot}{A function that will be called after the plotting, allowing
the user to add more to the plot.}

\item{ylab}{label for the y-axis}

\item{xaxis}{Logical, whether to add x-axis labels to plot (if FALSE can be manually created by user with call to addToPlot)}

\item{yaxis}{Logical, whether to add y-axis labels to plot (if FALSE can be manually created by user with call to addToPlot)}

\item{xlab}{label for the x-axis}

\item{...}{arguments to be passed to the individual plot commands (Will be
sent to all plot commands)}
}
\value{
This function creates a plot and does not return anything to the user.
}
\description{
Plotting splines
}
\details{
If \code{data} is NULL, the data plotted will be from
  \code{assay(object)}, after log-transformation if
  \code{log_transform(object)=TRUE}.

If \code{centroid} is missing, then splines will be estimated (per
  group) for the the data in \code{data} -- separately for each row of
  \code{data}. If \code{centroid} is provided, this data will be used to plot
  a spline function, and this same spline will be plotted for each row of
  \code{data}. This is useful, for example, in plotting cluster centroids
  over a series of genes.

If the user set \code{log_transform=TRUE} in the creation of the
  \code{Moanin} object, the data will be log transformed before plotting and
  calculating the spline fits.
}
\examples{
# First, load some data and create a moanin model
data(exampleData)
moanin <- create_moanin_model(data=testData,meta=testMeta, 
   degrees_of_freedom=6)

# The moanin model contains all the information for plotting purposes. The
# plot_splines_data will automatically fit the splines from the
# information contained in the moanin model
genes <- c("NM_001042489", "NM_008725")
plot_splines_data(moanin, subset_data=genes,
mfrow=c(2, 2))
# By default, same axis for all genes. Can change with 'simpleY=FALSE'
plot_splines_data(moanin, subset_data=genes,
   smooth=TRUE, mfrow=c(2,2), simpleY=FALSE)   

# The splines can also be smoothed
plot_splines_data(moanin, subset_data=genes,
   smooth=TRUE, mfrow=c(2, 2))
# You can provide different data (on same subjects),
# instead of data in moanin object
# (in which case moanin just provides grouping information)
plot_splines_data(moanin, data=1/assay(moanin), subset_data=genes,
   smooth=TRUE, mfrow=c(2, 2))
   
# You can also provide data to use for fitting splines to argument  
# "centroid". This is helpful for overlaying centroids or predicted data
# Here we do a silly example, just to demonstrate syntax, 
# where we use the data from the first gene as our centroid to fit a
# spline estimate, but plot data from genes 3-4
plot_splines_data(moanin, centroid=assay(moanin[1,]), subset_data=3:4,
   smooth=TRUE, mfrow=c(2,2))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/splines.R
\name{fit_splines}
\alias{fit_splines}
\title{Fit splines to each gene of data matrix, used by DE_timecourse}
\usage{
fit_splines(design_matrix, data, weights = NULL)
}
\arguments{
\item{design_matrix}{design matrix (containing evaluated basis matrix, but potentially other variables) to fit}

\item{data}{a matrix of data to fix splines to}

\item{weights}{A matrix of weights, of the same dimension as \code{data}.}
}
\value{
matrix of the coefficients for each basis function, each row of the
  matrix containing the coefficients for the corresponding gene in
  \code{data}.
}
\description{
Fit splines to each gene of data matrix, used by DE_timecourse
}
\details{
Needed to allow for fitting weights, in which case for loop over every row/gene. Otherwise, just a call to lm.fit.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DE_timecourse.R
\name{DE_timecourse}
\alias{DE_timecourse}
\alias{DE_timecourse,Moanin-method}
\title{Run spline models and test for DE of contrasts.}
\usage{
\S4method{DE_timecourse}{Moanin}(
  object,
  contrasts,
  center = FALSE,
  statistic = c("ftest", "lrt"),
  use_voom_weights = TRUE
)
}
\arguments{
\item{object}{An object of class \code{\link{Moanin}}, an object containing
all related information for time course data and the splines model that
will be used (if applicable). See \code{\link{create_moanin_model}} for
more details.}

\item{contrasts}{Contrasts, either provided as a vector of strings, or a
matrix of contrasts coefficients obtained using
\code{\link[limma]{makeContrasts}} from the package \code{limma}. If given
as a character string, will be passed to \code{\link[limma]{makeContrasts}}
to be converted into such a matrix.}

\item{center}{boolean, whether to center the data matrix}

\item{statistic}{Which test statistic to use, a likelihood ratio statistic or
a F-test.}

\item{use_voom_weights}{boolean, optional, default: TRUE. 
Whether to use voom weights. See details.}
}
\value{
A \code{data.frame} with two columns for each of the contrasts given
  in \code{contrasts}, corresponding to the raw p-value of the contrast for
  that gene (\code{_pval}) and the adjusted p-value (\code{_qval}). The
  adjusted p-values are FDR-adjusted based on the Benjamini-Hochberg method,
  as implemented in \code{\link[stats]{p.adjust}}. The adjustment is done
  across all p-values for all contrasts calculated.
}
\description{
Run spline models and test for DE of contrasts.
}
\details{
The implementation of the spline fit and the calculation of p-values
  was based on code from \code{\link[edge]{edge}}, and expanded to enable
  handling of comparisons of groups via contrasts. The code assumes that the \code{Moanin} object was created via either a formula or a basis where a different spline was fit for each \code{group_variable} and thus the contrasts are comparisons of those spline fits. If the \code{Moanin} object was created via user-provided basis matrix or formula, then the user should take a great deal of caution in using this code, as the degrees of freedom for the tests of significance cannot be verified to be correct.

If \code{use_voom_weights=TRUE}, then before fitting splines to each gene,
voom weights are calculated from \code{assay(object)}:
\preformatted{
   y <- edgeR::DGEList(counts=assay(object))
   y <- edgeR::calcNormFactors(y, method="upperquartile")
   v <- limma::voom(y, design, plot=FALSE)
   weights <- v$weights
}
The design matrix for the voom weights is based on the formula
\code{~Group + Timepoint +0}
where Group and Timepoint are replaced with the user-defined values where appropriate. 
These weights are given to the \code{lm.fit} which fits the spline coefficients.
This workflow assumes that the input to the \code{Moanin} object were counts.

If the user set \code{log_transform=TRUE} in the creation of the
  \code{Moanin} object, the splines will be fit to the log of the input data,
  and not directly to the input data. This is independent of whether the user
  chooses \code{use_voom_weights}.
}
\examples{
data(exampleData)
moanin <- create_moanin_model(data=testData, meta=testMeta)
deTimecourse=DE_timecourse(moanin, 
   contrasts="K-C", use_voom_weights=FALSE)
head(deTimecourse)
}
\seealso{
\code{\link[limma]{makeContrasts}}, \code{\link{create_moanin_model}},
  \code{\link{DE_timepoints}}, \code{\link[edge]{edge}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/splines.R
\name{fit_predict_splines}
\alias{fit_predict_splines}
\title{Get fitted values for splines for each gene}
\usage{
fit_predict_splines(data, moanin_model, meta_prediction = NULL)
}
\arguments{
\item{data}{a matrix with data (required doesn't pull from moanin_model)}

\item{meta_prediction}{}
}
\value{
a matrix of the fitted y values, with dimensions the same as
  \code{data}
}
\description{
Get fitted values for splines for each gene
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cluster.R
\name{splines_kmeans_predict,Moanin-method}
\alias{splines_kmeans_predict,Moanin-method}
\alias{splines_kmeans_score_and_label}
\alias{splines_kmeans_predict}
\alias{splines_kmeans_score_and_label,Moanin-method}
\title{Assign score and labels from raw data}
\usage{
\S4method{splines_kmeans_predict}{Moanin}(
  object,
  kmeans_clusters,
  data = NULL,
  method = c("distance", "goodnessOfFit"),
  ...
)

\S4method{splines_kmeans_score_and_label}{Moanin}(
  object,
  kmeans_clusters,
  data = NULL,
  proportion_genes_to_label = 0.5,
  max_score = NULL,
  previous_scores = NULL,
  rescale_separately = FALSE
)
}
\arguments{
\item{object}{the Moanin object that contains the basis functions used in
creating the clusters}

\item{kmeans_clusters}{List returned by \code{\link{splines_kmeans}}}

\item{data}{the data to predict. If not given, will use \code{assay(object)}.
If given, the number of columns of \code{data} must match that of
\code{object}}

\item{method}{If "distance", predicts based on distance of data to kmeans
centroids. If "goodnessOfFit", is a wrapper to
\code{splines_kmeans_score_and_label}, assigning labels based on goodness
of fit, including any filtering.}

\item{...}{arguments passed to \code{splines_kmeans_score_and_label}}

\item{proportion_genes_to_label}{float, optional, default: 0.5
Percentage of genes to label. If max_score is provided, will label
genes that are either in the top `proportion_genes_to_label` or with a
score below `max_score`.}

\item{max_score}{optional, default: Null
When provided, will only label genes below that score. If NULL, ignore
this option.}

\item{previous_scores}{matrix of scores, optional. Allows user to give the
matrix scores results from a previous run of
\code{splines_kmeans_score_and_label}, and only redo the filtering (i.e. if
want to change \code{proportion_genes_to_label} without rerunning the
calculation of scores)}

\item{rescale_separately}{logical, whether to score separately within
grouping variable}
}
\value{
\code{splines_kmeans_predict} returns a vector giving the labels for
  the given data.

A list consisting of
\itemize{
\item{\code{labels}}{the label or cluster assigned to each gene based on the
cluster with the best (i.e. lowest) score, with no label given to genes that
do not have a score lower than a specified quantity}
\item{\code{scores}}{the matrix of size n_cluster x n_genes, containing for 
each gene and each cluster, the goodness of fit score}
\item{\code{score_cutoff}}{The required cutoff for a gene receiving an
assignment}
}
}
\description{
Assign score and labels from raw data
}
\examples{
data(exampleData)
moanin <- create_moanin_model(data=testData, meta=testMeta)
# Cluster on a subset of genes
kmClusters=splines_kmeans(moanin[1:50,],n_clusters=3)
# get scores on all genes
scores_and_labels <- splines_kmeans_score_and_label(object=moanin, kmClusters)
head(scores_and_labels$scores)
head(scores_and_labels$labels)
# should be same as above, only just the assignments
predictLabels1 <- splines_kmeans_predict(object=moanin, kmClusters, 
     method="goodnessOfFit")
# Instead use distance to centroid:
predictLabels2 <- splines_kmeans_predict(object=moanin, kmClusters, 
     method="distance")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GO_terms_analysis.R
\name{find_enriched_go_terms}
\alias{find_enriched_go_terms}
\alias{create_go_term_mapping}
\title{Find enriched GO terms}
\usage{
find_enriched_go_terms(
  assignments,
  gene_id_to_go,
  ontology = "BP",
  weighted = FALSE,
  node_size = 10
)

create_go_term_mapping(genes, gene_col = "refseq_mrna")
}
\arguments{
\item{assignments}{boolean named vector determining the gene subset to be
tested for enrichment of GO terms. The names of the vector should be the
gene names. Elements with TRUE will consist of the gene cluster.}

\item{gene_id_to_go}{List giving the Gene ID to GO object required for topGO
(see \code{\link[topGO]{topGOdata-class}}). 
\code{create_go_term_mapping} can construct such a list from a
data-frame.}

\item{ontology}{string, optional, default: BP. specficies which ontology to
use (passed to \code{ontology} argument in creating a new \code{topGOdata}
object). Can be 'BP', 'CC', or 'NF'. See
\code{\link[topGO]{topGOdata-class}}.}

\item{weighted}{boolean, optional, default: FALSE. Whether to use the
weighted algorithm or not in \code{\link[topGO]{runTest}}.}

\item{node_size}{integer, optional, default: 10. Consider only GO terms with
node_size number of genes, passed to \code{nodeSize} argument of
\code{\link[topGO]{topGOdata-class}}}

\item{genes}{dataframe, with two required columns. The first gives the gene
names, with column name by the argument \code{gene_col}. The other column
must be named "go_id" and give the genes GO id. Genes will have multiple GO
id that they map to, and each go mapping of a gene is a separate row. Thus
genes will be in multiple rows of the input.}

\item{gene_col}{the name of the column of the \code{genes} data frame that
contains the correct gene reference. By default, is "refseq_mrna".}
}
\value{
Returns results in the format of \code{\link[topGO]{GenTable}}.

\code{create_go_term_mapping} returns a list giving the gene to GO id
  in the format required by \code{\link[topGO]{topGOdata-class}}.
}
\description{
Find enriched GO terms

Create the Gene to GO Term mapping
}
\details{
\code{find_enriched_go_terms} is a wrapper for running a GO
 enrichment analysis via the package \code{topGO}. This function creates a
 \code{\link[topGO]{topGOdata-class}} object, runs the function
 \code{\link[topGO]{runTest}} to test for enrichment using the
 \code{statistic="fisher"} option, and then runs
 \code{\link[topGO]{GenTable}}. This function then does some post-processing
 of the results, returning only GO terms that satisfy:
 \enumerate{
 \item{BH adjusted p-values less than 0.05 using
 \code{\link[stats]{p.adjust}}}
 \item{GO terms are \emph{enriched}, i.e. the number of genes from the GO 
 term found in the subset is greater than expected}
 }
}
\examples{
data(exampleData)
head(testGenesGO) #gives the mapping of genes to GO
geneId2Go <- create_go_term_mapping(testGenesGO)
#create fake assignment of genes to group based on TRUE/FALSE values
inGroup=rep(FALSE,nrow(testData))
inGroup[1:10]=TRUE
names(inGroup) <- row.names(testData)
find_enriched_go_terms(inGroup, geneId2Go)
}
\seealso{
\code{\link{create_go_term_mapping}}, \code{\link[KEGGprofile]{find_enriched_pathway}}, \code{\link[topGO]{GenTable}}, \code{\link[topGO]{runTest}}, \code{\link[topGO]{topGOdata-class}}, \code{\link[stats]{p.adjust}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllHelper.R
\name{Moanin-methods}
\alias{Moanin-methods}
\alias{group_variable_name}
\alias{group_variable_name,Moanin-method}
\alias{group_variable_name<-,Moanin-method}
\alias{group_variable_name<-}
\alias{time_by_group_variable,Moanin-method}
\alias{time_by_group_variable}
\alias{group_variable,Moanin-method}
\alias{group_variable}
\alias{group_variable<-,Moanin-method}
\alias{group_variable<-}
\alias{time_variable_name,Moanin-method}
\alias{time_variable_name}
\alias{time_variable_name<-,Moanin-method}
\alias{time_variable_name<-}
\alias{time_variable,Moanin-method}
\alias{time_variable}
\alias{time_variable<-,Moanin-method}
\alias{time_variable<-}
\alias{degrees_of_freedom,Moanin-method}
\alias{degrees_of_freedom}
\alias{basis_matrix,Moanin-method}
\alias{basis_matrix}
\alias{spline_formula,Moanin-method}
\alias{spline_formula}
\alias{show,Moanin-method}
\alias{show}
\alias{[,Moanin,ANY,character,ANY-method}
\alias{[,Moanin,ANY,ANY,ANY-method}
\alias{[,Moanin,ANY,logical,ANY-method}
\alias{[,Moanin,ANY,numeric,ANY-method}
\alias{log_transform,Moanin-method}
\alias{log_transform}
\alias{get_log_data,Moanin-method}
\alias{get_log_data}
\title{Helper methods for the Moanin class}
\usage{
\S4method{group_variable_name}{Moanin}(object)

\S4method{group_variable_name}{Moanin}(object) <- value

\S4method{time_by_group_variable}{Moanin}(object)

\S4method{group_variable}{Moanin}(object)

\S4method{group_variable}{Moanin}(object) <- value

\S4method{time_variable_name}{Moanin}(object)

\S4method{time_variable_name}{Moanin}(object) <- value

\S4method{time_variable}{Moanin}(object)

\S4method{time_variable}{Moanin}(object) <- value

\S4method{degrees_of_freedom}{Moanin}(object)

\S4method{basis_matrix}{Moanin}(object)

\S4method{spline_formula}{Moanin}(object)

\S4method{show}{Moanin}(object)

\S4method{[}{Moanin,ANY,character,ANY}(x, i, j, ..., drop = TRUE)

\S4method{[}{Moanin,ANY,logical,ANY}(x, i, j, ..., drop = TRUE)

\S4method{[}{Moanin,ANY,numeric,ANY}(x, i, j, ..., drop = TRUE)

\S4method{log_transform}{Moanin}(object)

\S4method{get_log_data}{Moanin}(object)
}
\arguments{
\item{object}{An object of class \code{\link{Moanin}}, an object containing
all related information for time course data and the splines model that
will be used (if applicable). See \code{\link{create_moanin_model}} for
more details.}

\item{value}{replacement value}

\item{x}{\code{Moanin} object}

\item{i, j}{A vector of logical or integer subscripts, indicating the rows and
columns to be subsetted for \code{i} and \code{j}, respectively.}

\item{...}{arguments passed to subsetting}

\item{drop}{A logical scalar that is ignored.}
}
\value{
\code{group_variable_name} and \code{time_variable_name} return the 
name of the column containing the variable. \code{group_variable} and 
\code{time_variable} return the  actual variable.
}
\description{
This is a collection of helper methods for the Moanin
  class.
}
\details{
Note that when subsetting the data, the dendrogram information and
  the co-clustering matrix are lost.
}
\examples{
# Load some data
data(exampleData)
moanin = create_moanin_model(data=testData,meta=testMeta)
group_variable_name(moanin)
time_variable_name(moanin)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cluster_validation.R
\name{consensus_matrix}
\alias{consensus_matrix}
\title{Compute consensus matrix from labels}
\usage{
consensus_matrix(labels, scale = TRUE)
}
\arguments{
\item{labels}{a matrix with each column corresponding to a the results of a
single clustering routine. Each column should give the cluster assignment
FIXME: What is the required format of entries??}

\item{scale}{boolean, optional, default: TRUE. Whether to rescale the
resulting consensus matrix so that entries correspond to proportions.}
}
\value{
a symmetric matrix of size NxN, where N is the number of rows of the
 input matrix \code{labels}. Each i,j entry of the matrix corresponds the
 number of times the two rows were in the same cluster across the clusterings
 (\code{scale=FALSE}) or the proportion of clustering that the two rows are
 in the same cluster (\code{scale=TRUE}).
}
\description{
Compute consensus matrix from labels
}
\examples{
data(exampleData)
moanin <- create_moanin_model(data=testData,meta=testMeta)
#small function to run splines_kmeans on subsample of 50 genes
subsampleCluster<-function(){
   ind<-sample(1:nrow(moanin),size=50,replace=FALSE)
   km<-splines_kmeans(moanin[ind,],n_clusters=3)
   assign<-splines_kmeans_predict(moanin,km, 
      method="distance")
  }
kmClusters=replicate(10,subsampleCluster())
cm<-consensus_matrix(kmClusters)
heatmap(cm)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/discont_basis.R
\name{discont_basis}
\alias{discont_basis}
\title{Provides set of basis functions on either side of a time point, allowing for a discontinuity in the fitted functions}
\usage{
discont_basis(
  timepoints,
  discont_point,
  knots = NULL,
  dfPre = NULL,
  dfPost = dfPre,
  degree = 3,
  intercept = TRUE,
  type = c("ns", "bs")
)
}
\arguments{
\item{timepoints}{vector of numeric timepoints for which the splines basis will be evaluated}

\item{discont_point}{a single numeric value that represents where the discontinuity should be}

\item{knots}{passed to \code{ns} or \code{bs}. If not NULL, should give knots on either side of \code{discon_point} as single vector -- they will be separated in the call to \code{discon_point}}

\item{dfPre}{the df for the basis functions defined before the discontinuity point}

\item{dfPost}{the df for the basis functions defined after the discontinuity point}

\item{degree}{passed to \code{bs} (if applicable)}

\item{intercept}{Whether to include an intercept (vector of all 1s) for each side of the discontinuity. Note this is different than the argument \code{intercept} of either \code{bs} or \code{ns}, which is set to \code{FALSE}.}

\item{type}{either "ns" or "bs" indicating which splines basis function should be used.}
}
\description{
Provides set of basis functions on either side of a time point, allowing for a discontinuity in the fitted functions
}
\examples{
x<-seq(0,10,length=100)
basis<-discont_basis(x,discont_point=3, dfPre=3, dfPost=4, intercept=TRUE)
# Plot of the basis functions
par(mfrow=c(3,3))
for(i in 1:ncol(basis)){
   plot(x,basis[,i],type="l")
   abline(v=3,lty=2)
}
# Use it in a moanin_model object instead of ns/bs:
data(exampleData)
moanin <- create_moanin_model(data=testData, meta=testMeta,
    spline_formula=~Group:discont_basis(Timepoint,dfPre=3,
        dfPost=3,discont=20,intercept=TRUE)+0,
    degrees_of_freedom=6)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/splines.R
\name{rescale_values}
\alias{rescale_values}
\alias{rescale_values,Moanin-method}
\alias{rescale_values,NULL-method}
\alias{rescale_values,}
\alias{NULL-method}
\alias{rescale_values,missing-method}
\title{Rescales rows of data to be between 0 and 1}
\usage{
\S4method{rescale_values}{Moanin}(object, data = NULL, use_group = FALSE)

\S4method{rescale_values}{`NULL`}(object, data)

\S4method{rescale_values}{missing}(object, ...)
}
\arguments{
\item{object}{a object of class Moanin, only needed if choose to rescale by
grouping variable in the moanin object. If NULL, then data will be rescaled
jointly across all observations.}

\item{data}{The matrix to rescale by row. If NULL, and \code{object} is
given, data will be taken as \code{assay(object)} Each row should
correspond to a gene or a centroid, and columns to samples.}

\item{use_group}{If true, then the data will be rescaled such that, for each
row, all values associated to each group (defined by grouping variable of
\code{object}) is between 0 and 1. For example, if column}

\item{...}{arguments passed to the matrix or Moanin method.}
}
\value{
rescaled y, such that for each row, the values are comprised between
  0 and 1. Note that if \code{use_group=TRUE} and \code{object} is not NULL,
  the values associated to the columns of unique values of the grouping
  variable of \code{object} will be rescaled separately.
}
\description{
Rescales rows of data to be between 0 and 1
}
\details{
If the user set \code{log_transform=TRUE} in the creation of the
  \code{Moanin} object, the data will be log transformed before rescaling
}
\examples{
data(exampleData)
moanin <- create_moanin_model(data=testData, meta=testMeta)
# Can rescale data in Moanin object
allData <- rescale_values(moanin)
# Or provide different data and/or rescale within grouping variable
smallData <- rescale_values(moanin, data=testData[1:10,], use_group=TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/splines.R
\name{create_meta_prediction}
\alias{create_meta_prediction}
\title{Create prediction meta data from splines model}
\usage{
create_meta_prediction(moanin_model, num_timepoints = 100)
}
\arguments{
\item{num_timepoints}{integer, optional, default: 100.
Number of timepoints to use for the prediction metadata}
}
\value{
moanin_model
}
\description{
Create prediction meta data from splines model
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{Moanin-class}
\alias{Moanin-class}
\alias{Moanin}
\alias{create_moanin_model,DataFrame-method}
\alias{create_moanin_model}
\alias{create_moanin_model,data.frame-method}
\alias{create_moanin_model,matrix-method}
\alias{create_moanin_model,SummarizedExperiment-method}
\title{Class Moanin}
\usage{
\S4method{create_moanin_model}{DataFrame}(data, meta, ...)

\S4method{create_moanin_model}{data.frame}(data, ...)

\S4method{create_moanin_model}{matrix}(data, meta, ...)

\S4method{create_moanin_model}{SummarizedExperiment}(
  data,
  spline_formula = NULL,
  basis_matrix = NULL,
  group_variable_name = "Group",
  time_variable_name = "Timepoint",
  degrees_of_freedom = NULL,
  log_transform = FALSE,
  drop_levels = TRUE
)
}
\arguments{
\item{data}{The input data. Can be a \code{\link{SummarizedExperiment}} class,
or matrix/data.frame. If the input data is a \code{matrix} or
\code{data.frame}, then the user must also provide input to the \code{meta}
argument, which will be transformed into \code{colData} of the resulting
\code{Moanin} object}

\item{meta}{Meta data on the samples (columns) of the \code{data} argument.
Must be given f input \code{data} is a \code{matrix} or \code{data.frame}.
If input is \code{SummarizedExperiment}, this argument is ignored.}

\item{...}{arguments passed from methods to the \code{SummarizedExperiment}
method.}

\item{spline_formula}{formula object, optional, default: NUlL. Used to
construct splines from the data in \code{meta}. See details.}

\item{basis_matrix}{matrix, optional, default: NULL. A basis matrix, where
each row corresponds to the evaluation of a sample on the basis function
(thus one column for each basis function).}

\item{group_variable_name}{A character value giving the column that
corresponds to the grouping variable to test for DE. By default "Group"}

\item{time_variable_name}{A character value giving the column that corresponds
to the time variable. By default "Timepoint".}

\item{degrees_of_freedom}{int, optional. Number of degrees of freedom to use
if neither the basis_matrix nor the spline_formula is provided. If not
provided by the user, internally will be set to 4}

\item{log_transform}{whether the data should be log-transformed by certain
methods (see \code{\link{splines_kmeans}})}

\item{drop_levels}{Logical, whether to perform \code{\link{droplevels}} on the
grouping variable (i.e. remove empty levels)}
}
\value{
An object of class \code{Moanin}
}
\description{
\code{Moanin} is a class that extends
  \code{SummarizedExperiment} and is used to store the additional spline
  basis and meta data for timecourse analysis.

In addition to the slots of the \code{SummarizedExperiment}
class, the \code{Moanin} object has the additional slots described
in the Slots section.

There are several methods implemented for this class. The most
  important methods have their own help page. Simple helper methods are
  described in the Methods section below. For a comprehensive list of methods
  specific to this class see the Reference Manual.

The constructor \code{create_moanin_model} creates an object of
 the class \code{Moanin}.
}
\details{
If neither \code{spline_formula} nor \code{basis_matrix} is given,
 then by default, the function will create a basis matrix based on the
 formula: \preformatted{spline_formula = ~Group:ns(Timepoint, df=4) + Group +
 0}

Note that the meta data will have levels dropped (via 
\code{droplevels}).

Input to \code{data} that is given as a class \code{DataFrame} or
 \code{data.frame} will be converted to class \code{matrix}. The reason
 for this is that use of a \code{data.frame} creates errors in taking duplicate rows/columns of \code{SummarizedExperiment}, as in bootstrapping. 
 Users who absolutely want the object to hold a
 object that is not a matrix can construct a \code{SummarizedExperiment} object
 (which will not convert the input into a \code{matrix}), and
 use this as input to \code{create_moanin_model}.
}
\section{Slots}{

\describe{
\item{\code{time_variable_name}}{character value giving the column in \code{colData}
that defines the time variable (must be of class \code{numeric})}

\item{\code{group_variable_name}}{character value giving the column in \code{colData}
that defines the grouping variable (must be of class \code{factor})}

\item{\code{basis_matrix}}{A basis matrix, where each row corresponds to the
evaluation of a sample on the basis function (thus one column for each
basis function).}

\item{\code{spline_formula}}{a formula. The formula used in creating the basis matrix}

\item{\code{degrees_of_freedom}}{a numeric integer. Number of degrees of freedom used
in creating basis matrix. If NULL, degrees of freedom is not known (usually
if user provided basis without degrees of freedom)}

\item{\code{log_transform}}{logical, whether to log-transform the data for certain
methods}
}}

\examples{
# Load some data
data(exampleData)

# Use the default options
moanin = create_moanin_model(data=testData,meta=testMeta)
moanin

# Change the number of degrees of freedom
moanin = create_moanin_model(data=testData,meta=testMeta, 
   degrees_of_freedom=6)
moanin
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cluster.R
\name{splines_kmeans}
\alias{splines_kmeans}
\alias{splines_kmeans,Moanin-method}
\title{Performs splines clustering using K-means}
\usage{
\S4method{splines_kmeans}{Moanin}(
  object,
  n_clusters = 10,
  init = "kmeans++",
  n_init = 10,
  max_iter = 300,
  random_seed = .Random.seed[1],
  fit_splines = TRUE,
  rescale = TRUE
)
}
\arguments{
\item{object}{An object of class \code{\link{Moanin}}, an object containing
all related information for time course data and the splines model that
will be used (if applicable). See \code{\link{create_moanin_model}} for
more details.}

\item{n_clusters}{int optional, default: 10}

\item{init}{["kmeans++", "random", "optimal_init"]}

\item{n_init}{int, optional, default: 10
Number of initialization to perform.}

\item{max_iter}{int, optional, default: 300
Maximum number of iteration to perform}

\item{random_seed}{int, optional, default: NULL. 
Passed to argument \code{seed} in  \code{\link[ClusterR]{KMeans_rcpp}}. 
If NULL (default), set to \code{.Random.seed[1]}.}

\item{fit_splines}{boolean, optional, default: TRUE
Whether to fit splines or not.}

\item{rescale}{boolean, optional, default: TRUE
Whether to rescale the data or not.}
}
\value{
A list in the format returned by \code{\link[ClusterR]{KMeans_rcpp}},
   with the following elements added or changed:
\itemize{
\item{\code{centroids}}{ The centroids are rescaled so that they range from
0-1}
\item{\code{fit_splines}}{ Logical, the value of \code{fit_splines} given to the
function }
\item{\code{rescale}}{ The value of \code{rescale} given to the function }
}
}
\description{
Performs splines clustering using K-means
}
\details{
If \code{\link{Moanin}} object's slot has log_transform=TRUE, then the data will be transformed by the function log(x+1) before applying splines and clustering.
}
\examples{
data(exampleData)
# Use the default options
moanin <- create_moanin_model(data=testData, meta=testMeta)
out <- splines_kmeans( moanin,n_clusters=5)
table(out$clusters)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/splines.R
\name{pvalues_fisher_method}
\alias{pvalues_fisher_method}
\title{Fisher's method to combine pvalues}
\usage{
pvalues_fisher_method(pvalues)
}
\arguments{
\item{pvalues}{a matrix of pvalues, with columns corresponding to different
tests or sources of p-values, and rows corresponding to the genes from
which the p-values come.}
}
\value{
a vector of p-values, one for each row of \code{pvalues}, that is the
  result of Fisher's combined probability test applied to the p-values in
  that row.
}
\description{
Combines all p-values per rows.
}
\examples{
data(exampleData)
moanin <- create_moanin_model(data=testData,meta=testMeta)
contrasts <- create_timepoints_contrasts(moanin,"C", "K")
deTimepoints=DE_timepoints(moanin, 
  contrasts=contrasts, use_voom_weights=FALSE)
fisherPval=pvalues_fisher_method(
  deTimepoints[,grep("pval",colnames(deTimepoints))])
head(fisherPval)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DE_timecourse.R, R/validation.R
\name{expand_contrast}
\alias{expand_contrast}
\alias{internal}
\alias{check_data_meta}
\alias{check_is_2d}
\alias{is_contrasts}
\title{Internal Validation Checks}
\usage{
expand_contrast(moanin_model, contrast_vector)

check_data_meta(data, object)

check_is_2d(X)

is_contrasts(contrasts, moanin_model)
}
\value{
Does not return anything. Only hits errors if there are problems.

\code{is_contrasts} returns the contrasts, with any corrections.
}
\description{
Will check that the contrasts provided are indeed contrasts. contrasts are
 expected to be either a vector of string or a matrix containing the
 contrasts coefficients.
}
\details{
If a vector of string is provided, the function will call
 limma::makeContrast in order to obtain the contrasts coefficients.

If a contrasts matrix is provided, it will perform a number of
  checks on the contrasts matrix to make sure it contains the number of rows
  expected, and that each contrast indeed sums to 0.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cluster_validation.R
\name{plot_cdf_consensus}
\alias{plot_cdf_consensus}
\alias{get_auc_similarity_scores}
\alias{plot_model_explorer}
\title{Evaluate the consensus between sets of clusterings}
\usage{
plot_cdf_consensus(labels)

get_auc_similarity_scores(labels, method = c("consensus", "nmi"))

plot_model_explorer(labels, colors = rainbow(length(labels)))
}
\arguments{
\item{labels}{a list. Each element of
the list is a matrix that gives the results of a clustering routine in each
column (see \code{\link{consensus_matrix}}). Usually each column would be
the result of running the clustering on a subsample or bootstrap resample
of the data.}

\item{method}{method for calculation of similarity for the AUC measure, one
of "consensus" or "nmi". See details.}

\item{colors}{a vector of colors, of length equal to the length of
\code{labels}}
}
\value{
\code{plot_cdf_consensus} invisibily returns list of the upper
  triangle values, with the list of same length as that of \code{labels}.

\code{get_auc_similarity_scores} returns a vector, equal to length
  of the list \code{labels}, giving the AUC value for each element of
  \code{labels}.

This function is a plotting function does not return anything
}
\description{
Methods for evaluating the consensus between sets of
  clusterings, usually in the context of subsetting of the data or different
  numbers of clusters.
}
\details{
For each element of the list \code{labels},
  \code{plot_cdf_consensus} calculates the consensus between the clusterings
  in the matrix, i.e. the number of times that pairs of rows are in the same
  cluster for different clusterings (columns) of the matrix using the
  \code{\link{consensus_matrix}} function. Then the set of values (the N(N-1)
  values in the upper triangle of the matrix), are converted into a cdf
  function and plotted.

For each set of clusterings given by \code{labels} (i.e. for each
  matrix \code{M} which is an element of the list \code{labels})
  \code{get_auc_similarity_scores} calculates a pairwise measure of
  similarity between the columns of \code{M}. These pairwise scores are
  plotted against their rank, and the final AUC measure is the area under
  this curve.

For method "consensus", the pairwise measure is given by calculating
  the consensus matrix using \code{\link{consensus_matrix}} with
  \code{scale=FALSE}. The consensus matrix is divided by the max of \code{M}.

For method "nmi", the pairwise value is the NMI value between each
  pair of columns of the matrix of clusterings using the
  \code{\link[NMI]{NMI}} function.
}
\examples{
data(exampleData)
moanin <- create_moanin_model(data=testData,meta=testMeta)
#small function to run splines_kmeans on subsample of 50 genes
subsampleCluster<-function(){
   ind<-sample(1:nrow(moanin),size=50)
   km<-splines_kmeans(moanin[ind,],n_clusters=3)
   assign<-splines_kmeans_score_and_label(moanin, km, 
       proportion_genes_to_label=1.0)$label
}
kmClusters1=replicate(10,subsampleCluster())
kmClusters2=replicate(10,subsampleCluster())
# Note, because of the small number of replicates (10), 
# these plots are not representative of what to expect.
out<-plot_cdf_consensus(labels=list(kmClusters1,kmClusters2)) 
get_auc_similarity_scores(list(kmClusters1,kmClusters2))
plot_model_explorer(list(kmClusters1,kmClusters2))
}
\seealso{
\code{\link{consensus_matrix}}, \code{\link[NMI]{NMI}},
  \code{\link{plot_cdf_consensus}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exampleData.R
\docType{data}
\name{exampleData}
\alias{exampleData}
\alias{testData}
\alias{testMeta}
\alias{testGenesGO}
\title{Small data set for running examples}
\format{
Three objects are loaded, a data frame of expression of 500 genes by
  84 samples (\code{testData}), a data frame with meta information on those
  84 samples (\code{testMeta}), and a data frame giving the GOID of the genes
  in \code{testData}.
}
\description{
Small data set for running examples
}
\details{
This data is a subset of the full time course data available as
  \code{\link[timecoursedata]{shoemaker2015}} and is only provided for the
  purpose of running examples, and not for biological meaning. Users should
  refer to the full data set.

The rownames of \code{testData} are RefSeq.
}
\examples{
#code used to create data:
\dontrun{
library(timecoursedata)
data(shoemaker2015)
testData<-shoemaker2015$data[1:500,]
whSamples<-which(shoemaker2015$meta$Group \%in\% c("C","K","M"))
testData<-testData[,whSamples]
testMeta<-droplevels(shoemaker2015$meta[whSamples,])
library(biomaRt)
ensembl = useMart("ensembl")
ensembl = useMart("ensembl")
ensembl = useDataset("mmusculus_gene_ensembl", mart=ensembl)
testGenesGO = getBM(attributes=c("go_id", "refseq_mrna"),
              values=rownames(testData),
              filters="refseq_mrna",
              mart=ensembl)
save(list=c("testData","testMeta","testGenesGO"),file="data/exampleData.rda")
}
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DE_timepoints.R
\name{perWeek_barplot}
\alias{perWeek_barplot}
\title{Creates barplot of results of per-timepoint comparison}
\usage{
perWeek_barplot(
  de_results,
  type = c("qval", "pval"),
  labels = NULL,
  threshold = 0.05,
  xlab = "Timepoint",
  ylab = "Number of DE genes",
  main = "",
  ...
)
}
\arguments{
\item{de_results}{results from \code{\link{DE_timepoints}}}

\item{type}{type of p-value to count ("qval" or "pval")}

\item{labels}{labels to give each bar}

\item{threshold}{cutoff for counting gene as DE}

\item{xlab}{x-axis label}

\item{ylab}{y-axis label}

\item{main}{title of plot}

\item{...}{arguments passed to \code{\link{barplot}}}
}
\value{
This is a plotting function, and returns (invisibly) the results of 
  \code{\link{barplot}}
}
\description{
Creates barplot of results of per-timepoint comparison
}
\details{
\code{create_timepoints_contrasts} creates the needed contrasts for
  comparing two groups for every timepoint in the format needed for
  \code{DE_timepoints} (i.e. \code{\link[limma]{makeContrasts}}, to which the
  contrasts are ultimately passed). The time points are determined by the
  meta data in the \code{moanin_object} provided by the user.
}
\examples{
data(exampleData)
moanin <- create_moanin_model(data=testData, meta=testMeta)
contrasts <- create_timepoints_contrasts(moanin, "C", "K")
deTimepoints <- DE_timepoints(moanin, 
    contrasts=contrasts, use_voom_weights=FALSE)
perWeek_barplot(deTimepoints)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalization.R
\name{expression_filtering}
\alias{expression_filtering}
\title{Utility function to filter out low-expressed genes}
\usage{
expression_filtering(counts, min_counts = 20, min_samples = 3)
}
\arguments{
\item{counts}{n by p matrix containing the count data.}

\item{min_counts}{integer, corresponding to the minimum number of counts for
the gene to be considered expressed in a sample. Default is 20.}

\item{min_samples}{integer, corresponding to the minimum of samples for a
gene to be expressed to be included in downstream analyses.}
}
\value{
The filtered counts matrix
}
\description{
Utility function to filter out low-expressed genes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DE_timepoints.R
\name{DE_timepoints}
\alias{DE_timepoints}
\alias{create_timepoints_contrasts}
\alias{DE_timepoints,Moanin-method}
\alias{create_timepoints_contrasts,Moanin-method}
\title{Fit weekly differential expression analysis}
\usage{
\S4method{DE_timepoints}{Moanin}(object, contrasts, add_factors = NULL, use_voom_weights = TRUE)

\S4method{create_timepoints_contrasts}{Moanin}(
  object,
  group1,
  group2 = NULL,
  type = c("per_timepoint_group_diff", "per_group_timepoint_diff",
    "group_and_timepoint_diff"),
  timepoints = sort(unique(time_variable(object))),
  timepoints_before = head(sort(timepoints), -1),
  timepoints_after = tail(sort(timepoints), -1),
  format = c("vector", "data.frame")
)
}
\arguments{
\item{object}{An object of class \code{\link{Moanin}}, an object containing
all related information for time course data and the splines model that
will be used (if applicable). See \code{\link{create_moanin_model}} for
more details.}

\item{contrasts}{Contrasts, either provided as a vector of strings, or a
matrix of contrasts coefficients obtained using
\code{\link[limma]{makeContrasts}} from the package \code{limma}. If given
as a character string, will be passed to \code{\link[limma]{makeContrasts}}
to be converted into such a matrix.}

\item{add_factors}{A character vector of additional variables to add to the 
design. See details.}

\item{use_voom_weights}{boolean, optional, default: TRUE. 
Whether to use voom weights. See details.}

\item{group1}{First group to consider in making contrasts, character value
that must match a value of the grouping variable contained in
\code{moanin_model}.}

\item{group2}{Second group to consider in making contrasts, character value
that must match a value of the grouping variable contained in
\code{moanin_model}, unless type=="per_group_timepoint_diff", in which case
should be NULL (only \code{group1} is used in comparison)}

\item{type}{the type of contrasts that should be created. See details.}

\item{timepoints}{vector of timepoints to compare. Must be contained in the
\code{time_variable} of the \code{moanin} object.}

\item{timepoints_before}{for \code{type} equal to
\code{"per_group_timepoint_diff"} or, \code{"group_and_timepoint_diff"},
the set of timepoints to compare, see details. By default, taken from the
\code{timepoints} variable.}

\item{timepoints_after}{for \code{type} equal to
\code{"per_group_timepoint_diff"} or, \code{"group_and_timepoint_diff"},
the set of timepoints to compare, see details. By default, taken from the
\code{timepoints} variable.}

\item{format}{the choice of "vector" (the default) for
\code{create_timepoints_contrasts} returns just the character vector of
contrasts. If instead \code{format="data.frame"} then a data.frame is
return that identifies the timepoint and group comparisons involved in each
contrast. If this is the desired output, then the input to
\code{DE_timepoints} should be the column corresponding to the contrast.
See examples.}
}
\value{


\code{create_timepoints_contrasts}: a character vector with each
  element of the vector corresponding to a contrast to be compared.
}
\description{
Fit weekly differential expression analysis

Creates pairwise contrasts for all timepoints
}
\details{
By default the formula fitted for each gene is
\preformatted{   
   ~ Group*Timepoint +0
}
If the user gives values to \code{add_factors}, then the vector of character 
values given in \code{add_factors} will be \emph{added} to the default formula. 
So that \code{add_factors="Replicate"} will change the formula to
\preformatted{   
   ~ Group*Timepoint +0 + Replicate
}
This allows for a small amount of additional complexity to control 
for other variables. Users should work directly with limma for 
more complex models.

If \code{use_voom_weights=TRUE}, the data is given directly to limma
  via \code{assay(object)}. The specific series of
  calls is:
\preformatted{   
   y <- edgeR::DGEList(counts=assay(object))
   y <- edgeR::calcNormFactors(y, method="upperquartile")
   v <- limma::voom(y, design, plot=FALSE)
   v <- limma::lmFit(v) 
   }

If the user set \code{log_transform=TRUE} in the creation of the
  \code{Moanin} object, this will not have an impact in the analysis if
  \code{use_voom_weights=TRUE}. Only if \code{use_voom_weights=FALSE} will
  this matter, in which case the log of the input data will be given to a
  regular call to \code{limma}:
\preformatted{
   y<-get_log_data(object)
   v <- limma::lmFit(y, design)
}

\code{create_timepoints_contrasts} creates the needed contrasts for
  comparing groups or timepoints in the format needed for
  \code{DE_timepoints} (i.e. \code{\link[limma]{makeContrasts}}), to which the
  contrasts are ultimately passed. The time points and groups are determined
  by the levels of the \code{grouping_variable} and the values of
  \code{time_variable} in the \code{moanin_object} provided by the user.

Three different types of contrasts are created:
  \itemize{
 \item{"per_timepoint_group_diff"}{Contrasts that compare the groups within a
 timepoint}
 \item{"per_group_timepoint_diff"}{Contrasts that compare two timepoints
 within a group}
 \item{"group_and_timepoint_diff"}{Contrasts that compare the
  difference between two timepoints between two levels of the
  \code{group_variable} of the \code{Moanin} object. These are contrasts in
  the form (TP i - TP (i-1))[Group1] - (TP i - TP (i-1))[Group2].}
}
}
\examples{
data(exampleData)
moanin <- create_moanin_model(data=testData, meta=testMeta)
# compare groups within each timepoint
contrasts <- create_timepoints_contrasts(moanin,"C", "K",
   type="per_timepoint_group_diff")
head(contrasts)
deTimepoints=DE_timepoints(moanin, 
    contrasts=contrasts, use_voom_weights=FALSE)
head(deTimepoints)
# Control for replicate variable:
deTimepoints=DE_timepoints(moanin, 
    contrasts=contrasts, add_factors="Replicate",
    use_voom_weights=FALSE)
head(deTimepoints)

# compare adjacent timepoints within each group
contrastsDiff <- create_timepoints_contrasts(moanin,"C",
   type="per_group_timepoint_diff")
deDiffTimepoints=DE_timepoints(moanin, 
    contrasts=contrastsDiff,
    use_voom_weights=FALSE)
# provide the sets of timepoints to compare:
contrastsDiff2<-create_timepoints_contrasts(moanin,"C",
   timepoints_before=c(72,120),timepoints_after=c(168,168),
   type="per_group_timepoint_diff")
deDiffTimepoints2=DE_timepoints(moanin, 
    contrasts=contrastsDiff2,
    use_voom_weights=FALSE)

# Compare selected timepoints across groups. 
# This time we also return format="data.frame" which helps us keep track of
# the meaning of each contrast. 
contrastsGroupDiff<-create_timepoints_contrasts(moanin,"C", "K",
   timepoints_before=c(72,120),timepoints_after=c(168,168),
   type="group_and_timepoint_diff",format="data.frame")
head(contrastsGroupDiff)
deGroupDiffTimepoints=DE_timepoints(moanin, 
    contrasts=contrastsGroupDiff$contrasts,
    use_voom_weights=FALSE)
}
\seealso{
\code{\link[limma]{makeContrasts}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimate_lfc.R
\name{estimate_log_fold_change}
\alias{estimate_log_fold_change}
\alias{estimate_log_fold_change,Moanin-method}
\title{Estimates log fold change}
\usage{
\S4method{estimate_log_fold_change}{Moanin}(
  object,
  contrasts,
  method = c("timecourse", "sum", "max", "timely", "abs_sum", "abs_squared_sum", "min")
)
}
\arguments{
\item{object}{An object of class \code{\link{Moanin}}, an object containing
all related information for time course data and the splines model that
will be used (if applicable). See \code{\link{create_moanin_model}} for
more details.}

\item{contrasts}{The contrasts to consider}

\item{method}{method for calculating the log-fold change. See details.}
}
\value{
A data.frame giving the estimated log-fold change for each gene
  (row). For all methods except for "timely", the data frame will consist of
  one column for each value of the argument \code{contrasts}. For "timely"
  there will be one column for each timepoint and contrast combination.
}
\description{
Estimates log fold change
}
\details{
The following methods exist for calculating the log-fold change
  between conditions over time (default is "timecourse"):
\itemize{
\item{\code{timely}}{The log-fold change for each individual timepoint
(\eqn{lfc(t)})}
\item{\code{timecourse}}{The average absolute per-week fold-change,
multiplied by the sign of the average per-week fold-change.}
\item{\code{sum}}{Sum of per-week log fold change, over all timepoints}
\item{\code{max}}{Max of per-week log fold change, over all timepoints}
\item{\code{abs_sum}}{Sum of the absolute value of the per-week log fold
change, over all timepoints}
\item{\code{abs_squared_sum}}{Sum of the square value of the per-week log
fold change, over all timepoint}
\item{\code{min}}{Min of per-week log fold change, over all timepoints}
}

If the user set \code{log_transform=TRUE} in the creation of the
  \code{Moanin} object, the data will be log transformed before calculating
  the fold-change.
}
\examples{
data(exampleData)
moanin <- create_moanin_model(data=testData,meta=testMeta)
estsTimely <- estimate_log_fold_change(moanin, 
   contrasts=c("K-C"), method="timely")
head(estsTimely)
estsTimecourse <- estimate_log_fold_change(moanin, 
   contrasts=c("K-C"),method="timecourse")
head(estsTimecourse)
}
